/**sbus.h
 * Mobilygen Static bus driver for Linux kernels
 */

/* Copyright 2004 Mobilygen
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#ifndef __SBUS_H
#define __SBUS_H

#include <linux/ioctl.h>

typedef struct {
    unsigned long address;
    unsigned long value;
} sbus_ioaccess_t;

typedef enum {
    SBUSEM_LITTLE_ENDIAN=0,
    SBUSEM_BIG_ENDIAN=1
} SBUS_ENDIAN_MODE;

typedef struct {
    SBUS_ENDIAN_MODE device;    /* Endianess of the target */
    unsigned long wordsize;     /* Size of a word in bits */
} sbus_endianmode_t;

/* Buses arguments. Selected using the 4 LSB of the minor number */
#define SBUS_NBUSOFFS	0
#define SBUS_NBUSMASK	0xF
/* Access mode arg. Selected using bits 7-4 of the minor number
 * Access mode 0 => PIO
 * Access mode n>=1 => DMA channel n-1
 */
#define SBUS_ACCMOFFS	4
#define SBUS_ACCMMASK	0xF

/* sbus /proc entry */
#define SBUS_PROCDIR        "driver/sbus"
#define SBUS_PROCDIR_LEN    strlen(SBUS_PROCDIR)
#define SBUS_PROCDMA        (SBUS_PROCDIR "/dma")
#define SBUS_PROCLOG        (SBUS_PROCDIR "/log")

/* Number of locking groups
 * This is the maximum value +1 that you can transmit
 * as an argument to SBUS_IOCS_LOCK
 * If one fd has called lock with group x and another
 * process try to call lock with the same group x argument
 * then the second process will have to wait until first one 
 * has finished.
 * In such case if O_NONBLOCK is set then ioctl will return -EAGAIN
 * otherwise ioctl will wait until first fd has called SBUS_IOCS_UNLOCK
 */
#define SBUS_MAXGROUPS 5

#define SBUS_IOCTL_MAGIC    's'

#define SBUS_IOCW_BYTE      _IOW(SBUS_IOCTL_MAGIC,0,sbus_ioaccess_t)
#define SBUS_IOCW_SHORT     _IOW(SBUS_IOCTL_MAGIC,1,sbus_ioaccess_t)
#define SBUS_IOCW_LONG      _IOW(SBUS_IOCTL_MAGIC,2,sbus_ioaccess_t)
#define SBUS_IOCR_BYTE      _IOWR(SBUS_IOCTL_MAGIC,3,sbus_ioaccess_t)
#define SBUS_IOCR_SHORT     _IOWR(SBUS_IOCTL_MAGIC,4,sbus_ioaccess_t)
#define SBUS_IOCR_LONG      _IOWR(SBUS_IOCTL_MAGIC,5,sbus_ioaccess_t)
#define SBUS_IOCS_IOBASE    _IOW(SBUS_IOCTL_MAGIC,6,unsigned long)
#define SBUS_IOCS_TXSIZE    _IOW(SBUS_IOCTL_MAGIC,7,unsigned long)
#define SBUS_IOCS_ADINCR    _IOW(SBUS_IOCTL_MAGIC,8,unsigned char)
#define SBUS_IOCS_LOCK      _IOW(SBUS_IOCTL_MAGIC,9,unsigned char)
#define SBUS_IOCS_UNLOCK    _IO(SBUS_IOCTL_MAGIC,10)
#define SBUS_IOCS_ENDIANESS _IOW(SBUS_IOCTL_MAGIC,11,sbus_endianmode_t)


#endif /* ifndef __SBUS_H */
