/*******************************************************************************
* 
* The content of this file or document is CONFIDENTIAL and PROPRIETARY
* to Mobilygen Corporation.  It is subject to the terms of a
* License Agreement between Licensee and Mobilygen Corporation.
* restricting among other things, the use, reproduction, distribution
* and transfer.  Each of the embodiments, including this information and
* any derivative work shall retain this copyright notice.
* 
* Copyright 2007 Mobilygen Corporation.
* All rights reserved.
* 
* QuArc is a registered trademark of Mobilygen Corporation.
* 
*******************************************************************************/

/*******************************************************************************
* @(#) $Id: luashell.h 14716 2008-04-11 21:59:30Z dsalim $
*******************************************************************************/
#ifndef LUASHELL_H
#define LUASHELL_H

#include "lua.h"

typedef int (*LUAFUNCPTR) (lua_State *L);

typedef struct
{
    char *name;
    char *terseHelp;
    char *verboseHelp;
    LUAFUNCPTR funcptr;
} LUA_HANDLER;

int LuaShellInit(int argc, char *argv[]);
int LuaShellExecuteCommandLineChunk(char *chunk);
int LuaShellStartInteractive(char startupScript[][1024], int numScripts, FILE *interactive);
int LuaShellRegisterFunction(LUA_HANDLER *handler);
int LuaShellRegisterGlobalFunction(LUA_HANDLER *handler);
int LuaShellHelp(int type, char *cmd);

#define LUA_GLOBAL              1
#define LUA_SYSCTL_OBJ          2 
#define LUA_LVPP_OBJ            3 
#define LUA_AVOC_OBJ            4 
#define LUA_NRTVPP_OBJ          5
#define LUA_AVSN_OBJ            6
#define LUA_SAIN_OBJ            7
#define LUA_HVOUT_OBJ           8
#define LUA_WRITESTREAMER_OBJ   9
#define LUA_AVE_OBJ            10
#define LUA_READSTREAMER_OBJ  11

#define CHECKARGS1ONLY \
if (lua_gettop(L) != 1) \
{\
    printf("Command takes one argument only\n"); \
    return(0); \
}

#define CHECKARGS2ONLY \
if (lua_gettop(L) != 2) \
{\
    printf("Command takes two arguments only\n"); \
    return(0); \
}

#define CHECKARGS3ONLY \
if (lua_gettop(L) != 3) \
{\
    printf("Command takes three arguments only\n"); \
    return(0); \
}

#define CHECKARGS4ONLY \
if (lua_gettop(L) != 4) \
{\
    printf("Command takes four arguments only\n"); \
    return(0); \
}

#define CHECKARGS5ONLY \
if (lua_gettop(L) != 5) \
{\
    printf("Command takes five arguments only\n"); \
    return(0); \
}

#define CHECKARGS6ONLY \
if (lua_gettop(L) != 6) \
{\
    printf("Command takes six arguments only\n"); \
    return(0); \
}

#define CHECKARGS7ONLY \
if (lua_gettop(L) != 7) \
{\
    printf("Command takes seven arguments only\n"); \
    return(0); \
}

#define CHECKARGS1OR2 \
if (lua_gettop(L) != 1) \
{\
    printf("Command takes one or two arguments only\n"); \
    return(0); \
}

#define CHECKARGS1F \
CHECKARGS1ONLY \
if ((lua_gettop(L) == 1) && (!lua_isnumber(L, 1))) \
{\
    printf("Argument must be floating point\n"); \
    return(0); \
}

#define CHECKARGS1N \
CHECKARGS1ONLY \
if ((lua_gettop(L) == 1) && (!lua_isnumber(L, 1))) \
{\
    printf("Argument must be numeric\n"); \
    return(0); \
}

#define CHECKARGS2N \
CHECKARGS2ONLY; \
if ((lua_gettop(L) == 2) && ((!lua_isnteger(L, 1)) || (!lua_isnumber(L, 2)))) \
{\
    printf("Argument 1 and 2 must be numeric\n"); \
    return(0); \
}

#define CHECKARGS3N \
CHECKARGS3ONLY; \
if ((lua_gettop(L) == 3) && ((!lua_isnumber(L, 1)) || (!lua_isnumber(L, 2)) || (!lua_isnumber(L, 3)))) \
{\
    printf("Argument 1, 2 and 3 must be numeric\n"); \
    return(0); \
}

#define LUACHECKARGS4N \
CHECKARGS4ONLY; \
if ((lua_gettop(L)  == 4) && ((!lua_isnumber(L, 1)) || (!lua_isnumber(L, 2)) || (!lua_isnumber(L, 3)) || (!lua_isnumber(L, 4)))) \
{\
    printf("Argument 1, 2, 3, and 4 must be numeric\n"); \
    return(0); \
}

#define CHECKARGS5N \
CHECKARGS5ONLY; \
if ((lua_gettop(L) == 5) && ((!lua_isnumber(L, 1)) || (!lua_isnumber(L, 2)) || (!lua_isnumber(L, 3)) || (!lua_isnumber(L, 4)) || (!lua_isnumber(L, 5)))) \
{\
    printf("Argument 1, 2, 3, 4, and 5 must be numeric\n"); \
    return(0); \
}

#define CHECKARGS6N \
CHECKARGS6ONLY; \
if ((lua_gettop(L) == 6) && ((!lua_isnumber(L, 1)) || (!lua_isnumber(L, 2)) || (!lua_isnumber(L, 3)) || (!lua_isnumber(L, 4)) || (!lua_isnumber(L, 5)) || (!lua_isnumber(L, 6)))) \
{\
    printf("Argument 1, 2, 3, 4, 5 and 6 must be numeric\n"); \
    return(0); \
}

#define CHECKARGS7N \
CHECKARGS7ONLY; \
if ((lua_gettop(L) == 7) && ((!lua_isnumber(L, 1)) || (!lua_isnumber(L, 2)) || (!lua_isnumber(L, 3)) || (!lua_isnumber(L, 4)) || (!lua_isnumber(L, 5)) || (!lua_isnumber(L, 6)) || (!lua_isnumber(L, 7)))) \
{\
    printf("Argument 1, 2, 3, 4, 5, 6 and 7 must be numeric\n"); \
    return(0); \
}

#define CHECKARGS1S \
CHECKARGS1ONLY; \
if ((lua_gettop(L) == 1) && (!lua_isstring(L, 1))) \
{\
    printf("Argument 1 must be a string\n"); \
    return(0); \
}

#define CHECKARGS2S \
CHECKARGS2ONLY; \
if ((lua_gettop(L) == 2) && ((!lua_isstring(L, 1)) || (!lua_isstring(L, 2)))) \
{\
    printf("Argument 1 and 2 must be strings\n"); \
    return(0); \
}

#define CHECKARGS3S \
CHECKARGS3ONLY; \
if ((lua_gettop(L) == 3) && ((!lua_isstring(L, 1)) || (!lua_isstring(L, 2)) || (!lua_isstring(L, 3)))) \
{\
    printf("Argument 1, 2 and 3 must be strings\n"); \
    return(0); \
}

#define CHECKARGS4S \
CHECKARGS4ONLY; \
if ((lua_gettop(L) == 4) && ((!lua_isstring(L, 1)) || (!lua_isstring(L, 2)) || (!lua_isstring(L, 3)) || (!lua_isstring(L, 4)))) \
{\
    printf("Argument 1, 2, 3 and 4 must be strings\n"); \
    return(0); \
}

// check first arg is number, optional second arg also a number
#define CHECKARGS1OR2N \
if (((lua_gettop(L) == 1) && (lua_isnumber(L, 1))) || \
    ((lua_gettop(L) == 2) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2)))) \
{ \
} \
else \
{ \
    printf("Command takes 1 or 2 numeric arguments\n"); \
    return(0); \
}

// check first arg is number, optional second and third args also numbers
#define CHECKARGS1OR3N \
if (((lua_gettop(L) == 1) && (lua_isnumber(L, 1))) || \
    ((lua_gettop(L) == 3) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2)) && (lua_isnumber(L, 3)))) \
{ \
} \
else \
{ \
    printf("Command takes 1 or 3 numeric arguments\n"); \
    return(0); \
}

// check first arg is number, optional second, third and fourth args also numbers
#define CHECKARGS1OR4N \
if (((argc == 1) && (lua_isnumber(L, 1))) || \
    ((argc == 4) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2)) && (lua_isnumber(L, 3)) && (lua_isnumber(L, 4)))) \
{ \
} \
else \
{ \
    printf("Command takes 1 or 4 numeric arguments\n"); \
    return(0); \
}

// check 2 or 3 numeric args
#define CHECKARGS2OR3N \
if ((lua_gettop(L) == 0) || (lua_gettop(L) == 1) || (!lua_isnumber(L, 1)) || (!lua_isnumber(L, 2))) \
{ \
    printf("Command takes 2 or 3 numeric arguments\n"); \
    return(0); \
}\
if ((lua_gettop(L) == 2) && (!lua_isnumber(L, 2))) \
{ \
    printf("Argument 2 must be numeric\n"); \
    return(0); \
} \
if ((lua_gettop(L) != 2) && (lua_gettop(L) != 3)) \
{ \
    printf("Command takes two or three numeric arguments\n"); \
    return(0); \
}

// check 2 or 4 numeric args
#define CHECKARGS2OR4N \
if (((lua_gettop(L) == 2) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2))) || \
    ((lua_gettop(L) == 4) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2)) && (lua_isnumber(L, 3)) && (lua_isnumber(L, 4)))) \
{ \
} \
else \
{ \
    printf("Command takes 2 or 4 numeric arguments\n"); \
    return(0); \
}

// check 2 or 5 numeric args
#define CHECKARGS2OR5N \
if (((lua_gettop(L) == 2) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2))) || \
    ((lua_gettop(L) == 5) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2)) && (lua_isnumber(L, 3)) && (lua_isnumber(L, 4)) && (lua_isnumber(L, 5)))) \
{ \
} \
else \
{ \
    printf("Command takes 2 or 5 numeric arguments\n"); \
    return(0); \
}

// one, two or 5 arguments
#define CHECKARGS1OR2OR5N \
if (((lua_gettop(L) == 1) && (lua_isnumber(L, 1))) || \
    ((lua_gettop(L) == 2) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2))) || \
    ((lua_gettop(L) == 5) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2)) && (lua_isnumber(L, 3)) && (lua_isnumber(L, 4)) && (lua_isnumber(L, 5)) )) \
{ \
} \
else \
{ \
    printf("Command takes 1, 2 or 5 numeric arguments\n"); \
    return(0); \
}

// one, two or 3 arguments
#define CHECKARGS1OR2OR3N \
if (((lua_gettop(L) == 1) && (lua_isnumber(L, 1))) || \
    ((lua_gettop(L) == 2) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2))) || \
    ((lua_gettop(L) == 3) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2)) && (lua_isnumber(L, 3)) )) \
{ \
} \
else \
{ \
    printf("Command takes 1, 2 or 3 numeric arguments\n"); \
    return(0); \
}

// two, four or 5 arguments
#define CHECKARGS2OR4OR5N \
if  (((lua_gettop(L) == 2) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2))) || \
     ((lua_gettop(L) == 4) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2)) && (lua_isnumber(L, 3)) && (lua_isnumber(L, 4))) || \
     ((lua_gettop(L) == 5) && (lua_isnumber(L, 1)) && (lua_isnumber(L, 2)) && (lua_isnumber(L, 3)) && (lua_isnumber(L, 4)) && (lua_isnumber(L, 5)) )) \
{ \
} \
else \
{ \
    printf("Command takes 2, 4 or 5 numeric arguments\n"); \
    return(0); \
}

// check zero or one numeric argument
#define CHECKARGS0OR1N \
if ((lua_gettop(L) == 1) && (!lua_isnumber(L, 1))) \
{ \
    printf("Argument 1 must be numeric\n"); \
    return(0); \
} \
if ((lua_gettop(L) != 0) && (lua_gettop(L) != 1)) \
{ \
    printf("Command takes zero or one numeric argument\n"); \
    return(0); \
}

// check zero or one string argument
#define CHECKARGS0OR1S \
if ((lua_gettop(L) == 0) || \
    ((lua_gettop(L) == 1) && (lua_isstring(L, 1)))) \
{ \
} \
else \
{ \
    printf("Command takes zero or one string argument\n"); \
    return(0); \
}

#define CHECKARGS1SOR1S1N \
if (((lua_gettop(L) == 1) && (lua_isstring(L, 1))) || \
   ((lua_gettop(L) == 2) && (lua_isstring(L, 1)) && (lua_isnumber(L, 2)))) \
{ \
} \
else \
{ \
    printf("Command takes one string argument and an optional numeric argument\n"); \
    return(0); \
}

#define CHECKARGS1SOR1S2N \
if (((lua_gettop(L) == 1) && (lua_isstring(L, 1))) || \
   ((lua_gettop(L) == 3) && (lua_isstring(L, 1)) && (lua_isnumber(L, 2)) && (lua_isnumber(L, 3)) )) \
{ \
} \
else \
{ \
    printf("Command takes one string argument or one string with 2 numeric arguments\n"); \
    return(0); \
}

#define CHECKARGS4N1S \
if ((lua_gettop(L) != 5) || (!lua_isnumber(L, 1)) || (!lua_isnumber(L, 2)) || (!lua_isnumber(L, 3)) || (!lua_isnumber(L, 4)) || (!lua_isstring(L, 5))) \
{\
    printf("Command takes four numeric and one string arguments\n"); \
    return(0); \
}


#define CHECKARGS1SAND2NOR1S \
if (((lua_gettop(L) == 3) && (lua_isstring(L, 1)) &&(lua_isnumber(L, 2)) && (lua_isnumber(L, 3))) || \
   ((lua_gettop(L) == 1) && (lua_isstring(L, 1)))) \
{ \
} \
else \
{ \
    printf("Command takes one string argument and two optional numeric arguments\n"); \
    return(0); \
} \

#define CHECKARGS1N1S \
if ((lua_gettop(L) == 2) && (lua_isnumber(L, 1)) && (lua_isstring(L, 2))) \
{ \
} \
else \
{ \
    printf("Command takes one numeric argument and one string argument\n"); \
    return(0); \
}

#define CHECKARGS1S2N \
if ((lua_gettop(L) ==3) && (lua_isstring(L, 1)) &&(lua_isnumber(L, 2)) && (lua_isstring(L, 3))) \
{ \
} \
else \
{ \
    printf("Command takes one string argument and two numeric arguments\n"); \
    return(0); \
} 

#define CHECKARGS1F2N \
if ((lua_gettop(L) ==3) && (lua_isnumber(L, 1)) &&(lua_isnumber(L, 1)) && (lua_isnumber(L, 1))) \
{ \
} \
else \
{ \
    printf("Command takes one floating argument and two numeric arguments\n"); \
    return(0); \
}


#endif  /* LUASHELL_H */
/*******************************************************************************
* vi: set shiftwidth=4 tabstop=8 softtabstop=4 expandtab nosmarttab:
*******************************************************************************/
