/*******************************************************************************
* 
* The content of this file or document is CONFIDENTIAL and PROPRIETARY
* to Mobilygen Corporation.  It is subject to the terms of a
* License Agreement between Licensee and Mobilygen Corporation.
* restricting among other things, the use, reproduction, distribution
* and transfer.  Each of the embodiments, including this information and
* any derivative work shall retain this copyright notice.
* 
* Copyright 2006 Mobilygen Corporation.
* All rights reserved.
* 
* QuArc is a registered trademark of Mobilygen Corporation.
* 
*******************************************************************************/

/*******************************************************************************
* @(#) $Id: qmp4.h 12995 2008-01-02 21:45:19Z bgarrow $
*******************************************************************************/
#ifndef __QMP4_HH
#define __QMP4_HH

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#define DEFAULT(x)	=x
#else
#define DEFAULT(x)
typedef int bool;
#endif

/* QMP4 API types */
typedef void* QMP4FileHandle;
typedef uint32_t QMP4TrackId;
typedef uint32_t QMP4Timestamp;
/* track media types */
#define QMP4_MEDIA_TYPE_UNKNOWN 0x0
#define QMP4_AUDIO_TYPE_MP4A 0x1
#define QMP4_AUDIO_TYPE_MP2A 0x2
#define QMP4_AUDIO_TYPE_MP1A 0x3
#define QMP4_VIDEO_TYPE_H264 0x4
#define QMP4_SAMPLE_FLAGS_SYNC_POINT (0x01)
#define QMP4_SAMPLE_FLAGS_CONFIGURATION (0x02)
#define QMP4_SAMPLE_FLAGS_NEW_FRAGMENT (0x4)

/* Invalid values for API types */
#define QMP4_INVALID_FILE_HANDLE ((QMP4FileHandle)NULL)
#define QMP4_INVALID_TRACK_ID ((QMP4TrackId)0)

/* Macros to test for API type validity */
#define QMP4_IS_VALID_FILE_HANDLE(x) ((x) != QMP4_INVALID_FILE_HANDLE)
#define QMP4_IS_VALID_TRACK_ID(x) ((x) != QMP4_INVALID_TRACK_ID)

/* file operations */
/* set QMP4IOCB function pointers to override default file I/O */
typedef struct {
    int (*read) ( int fd, void *dest, int bytes );
    int (*write) ( int fd, const void *src, int bytes );
    int64_t (*seek) ( int fd, int64_t offset, int whence );
} QMP4IOCB;

/* movie operations */
/* set limit to 0 if no memory constraint, in seconds */
/* set nextSample function pointer to know sample information in advance */
typedef void (*QMP4NextSample) (void *arg, QMP4TrackId trackId, uint32_t size, uint32_t flags, QMP4Timestamp cts);
/* set getBuffer function pointer to read sample in smaller chunks */
typedef char *(*QMP4GetBuffer) (void *arg, int *size);
typedef struct {
    int fd;
    int limit;
    QMP4IOCB *io;
    QMP4NextSample nextSample;
    void           *nextSampleArg;
    QMP4GetBuffer getBuffer;
    void *getBufferArg;
} QMP4DInitParm;
QMP4FileHandle QMP4DInit(QMP4FileHandle handle, QMP4DInitParm parm, bool cacheQ);
QMP4FileHandle QMP4DInitName(QMP4FileHandle handle, QMP4DInitParm parm, const char* sourceFilename, bool cacheQ);
typedef enum {
    MODE_QMP4_FWDPLAY,
    MODE_QMP4_BWDPLAY,
    MODE_QMP4_FWDIPLAY,
    MODE_QMP4_BWDIPLAY,
} QMP4DMode;
typedef struct {
    uint32_t pts;
    QMP4DMode mode;
} QMP4OpenParm;
bool QMP4Open(QMP4FileHandle handle, QMP4OpenParm parm);
void QMP4Close(QMP4FileHandle handle);
void QMP4DExit(QMP4FileHandle handle);

/* track operations */
/* return 0 if failed */
uint32_t QMP4GetNumberOfTracks(
	QMP4FileHandle handle);
/* return QMP4_INVALID_TRACK_ID if failed, index from 0 to tracks-1 */
QMP4TrackId QMP4FindTrackId(
	QMP4FileHandle handle, 
	uint16_t index);
/* return track media type */
int QMP4GetTrackType(
    QMP4FileHandle handle, 
	QMP4TrackId trackId);
/* return 0 if failed */
uint32_t QMP4GetTrackTimeScale(
	QMP4FileHandle handle,
	QMP4TrackId trackId);
/* get video sequence parameter set */
bool QMP4GetSequenceParameterSet(
    QMP4FileHandle handle,
    QMP4TrackId trackId,
    uint32_t index,
    uint8_t* pData, 
	uint32_t* pSize);
/* get video picture parameter set */
bool QMP4GetPictureParameterSet(
    QMP4FileHandle handle,
    QMP4TrackId trackId,
    uint32_t index,
    uint8_t* pData, 
	uint32_t* pSize);
/* get audio specific info */
bool QMP4GetAudioSpecificInfo(
    QMP4FileHandle handle,
    QMP4TrackId trackId,
    uint8_t* pData, 
	uint32_t* pSize);

/* sample operations */
/* read the next sample for decoding.
   return sample description index or 0. */
uint32_t QMP4ReadSample(
    /* input parameters */
    QMP4FileHandle handle,
    /* output parameters */
    QMP4TrackId* pTrackId,
    uint8_t* pBytes, 
	uint32_t* pNumBytes,
    QMP4Timestamp* pCompositionTime,
    bool* pIsSyncSample);

/* misc */
bool QMP4GetMaxVideoGOP(int fdin, uint32_t *gop);

/* movie operations */
/* limit: 0 sec will dynamic alloc/rel memory otherwise created during init */
/* maxSamplesPerFragment: affect required memory &caller's buffer management */
/* set releaseBuffer function pointer to know sample buffer can be released */
typedef void (*QMP4ReleaseBuffer) (int addr, int size);
typedef struct {
    int fd;
    int limit;
    int maxSamplesPerFragment;
    QMP4IOCB *io;
    QMP4ReleaseBuffer releaseBuffer;
} QMP4MInitParm;
QMP4FileHandle QMP4MInit(QMP4FileHandle handle, QMP4MInitParm parm);
/* set mdat to 1 to create single media data */
typedef struct {
    int mdat;
} QMP4CreateParm;
/*
    support for chunk files ...
*/
int QMP4SetChunkSession(QMP4FileHandle handle, const char* filename, unsigned long maxSize) ;
typedef void (*chunkFunc_t)(const int audioFile, const unsigned long size, const unsigned long addr) ;
int QMP4SetChunkFunc(QMP4FileHandle handle, chunkFunc_t parm);
/*

*/
bool QMP4Create(QMP4FileHandle handle, QMP4CreateParm parm);
void QMP4Finish(QMP4FileHandle handle);
void QMP4MExit(QMP4FileHandle handle);

/* track operations */
/* add audio/video track, return trackId */
int QMP4AddTrack(QMP4FileHandle handle, int type);
/* set video time scale */
bool QMP4SetVideoTimeScale(QMP4FileHandle handle, int timescale, int delta);
/* set video format */
bool QMP4SetVideoFormat(QMP4FileHandle handle, int width, int height);
/* set audio format */
bool QMP4SetAudioFormat(QMP4FileHandle handle, int channels, int samplesize, int samplerate);
/* get video sequence parameter set */
bool QMP4SetSequenceParameterSet(
    QMP4FileHandle handle,
    QMP4TrackId trackId,
    uint32_t index,
    uint8_t* pData, 
	uint32_t size);
/* set video picture parameter set */
bool QMP4SetPictureParameterSet(
    QMP4FileHandle handle,
    QMP4TrackId trackId,
    uint32_t index,
    uint8_t* pData, 
	uint32_t size);
/* set audio specific info */
bool QMP4SetAudioSpecificInfo(
    QMP4FileHandle handle,
    QMP4TrackId trackId,
    uint8_t* pData, 
	uint32_t size);

/* sample operations */
/* write the next sample. sample buffer can be resued after new fragment */
bool QMP4WriteSample(
    /* input parameters */
    QMP4FileHandle handle,
    QMP4TrackId trackId,
    uint8_t* pBytes, 
	uint32_t numBytes,
    uint32_t flags,
    QMP4Timestamp compositionTime);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif  /* __QMP4_HH */
/*******************************************************************************
* vi: set shiftwidth=4 tabstop=8 softtabstop=4 expandtab nosmarttab:
*******************************************************************************/
