--[[

This script will encode 2 stereo channels simultaneously.  It also
has the option to split the 2 stereo channels into 4 mono channels.

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Audio Input
-- onboard = onboard audio, kite = kite daughtercard
AIN = AIN or 'onboard';

-- Audio sample rate
SAMPLERATE = SAMPLERATE or 16000;

-- Audio bitrate
ABITRATE = ABITRATE or 64000;

-- Controls the audio encoder type
--   "AAC-LC"
-- 	 "QMA"
-- 	 "PCM"
--   "G711" (defaults to mu-law) 
--   "G722"
--   "G726"
--   "G728"
AUDIO_ENCODER = AUDIO_ENCODER or "PCM";

-- Audio frame size (may be overridden depending on codec)
AUDIO_FRAME_SIZE = AUDIO_FRAME_SIZE or 256; 

-- Force check for sampling rate, number of channels and frame size
if (AUDIO_CODEC ==  Q_AUDENC_CFP_CODEC_Q726) or (AUDIO_CODEC ==  Q_AUDENC_CFP_CODEC_Q711)
   or (AUDIO_CODEC ==  Q_AUDENC_CFP_CODEC_Q728) then
   SAMPLERATE = 8000;
end
if (AUDIO_CODEC ==  Q_AUDENC_CFP_CODEC_QMA) then
   AUDIO_FRAME_SIZE=1152;
end


-- Split 2 stereo channels into 4 mono channels
SPLIT_STEREO_INPUT = SPLIT_STEREO_INPUT or 1;

-- 0 for nothing, 1 for input preview, 2 for audio splitter output
PREVIEW = PREVIEW or 1;

-- Audio output source
-- PREVIEW = 1: 
--     0 = AIN0, 1 = AIN1
-- PREVIEW = 2 and SPLIT_STEREO_INPUT = 1:
--     0/1 = AIN0 ch0/1, 2/3 = AIN1 ch0/1
AUDIO_OUTPUT_SRC = AUDIO_OUTPUT_SRC or 0;

-- An option to turn off audio encoding when set to 0
AUDIO_ENCODE_ON = AUDIO_ENCODE_ON or 1;

OUTPUTFILENAME = {};
OUTPUTFILENAME[0] = OUTPUTFILENAME[0] or "/media/sda1/aud/audenc0.qbx";
OUTPUTFILENAME[1] = OUTPUTFILENAME[1] or "/media/sda1/aud/audenc1.qbx";
OUTPUTFILENAME[2] = OUTPUTFILENAME[2] or "/media/sda1/aud/audenc2.qbx";
OUTPUTFILENAME[3] = OUTPUTFILENAME[3] or "/media/sda1/aud/audenc3.qbx";

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

if (AUDIO_ENCODER == "AAC-LC") then
    AUDIO_CODEC = Q_AUDENC_CFP_CODEC_QAC;
    AUDIO_FRAME_SIZE = 1024;
elseif (AUDIO_ENCODER == "QMA") then
   AUDIO_CODEC = Q_AVDEC_CFP_AUDIO_CODEC_QMA;
   ABITRATE = 64000;
   AUDIO_FRAME_SIZE = 1152;
elseif (AUDIO_ENCODER == "PCM") then
   AUDIO_CODEC =  Q_AVDEC_CFP_AUDIO_CODEC_QPCM;
elseif (AUDIO_ENCODER == "G711") then
    AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q711;
    SAMPLERATE = 8000;
    ABITRATE = 64000;
elseif (AUDIO_ENCODER == "G722") then
    AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q722;
    SAMPLERATE = 16000;
    ABITRATE = 64000;
elseif (AUDIO_ENCODER == "G726") then
    AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q726;
    SAMPLERATE = 8000;
    if (ABITRATE ~= 40000) and (ABITRATE ~= 32000) and (ABITRATE ~= 24000) and (ABITRATE ~= 16000) then
        ABITRATE = 40000; -- set usable default
    end
elseif (AUDIO_ENCODER == "G728") then
    AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q728;
    SAMPLERATE = 8000;
    ABITRATE = 64000;
else
    error("Invalid audio encoder " .. tostring(AUDIO_ENCODER))
end


if (SPLIT_STEREO_INPUT == 1) then
    AUDCH = 4;
    AUDENC_NUMCH = Q_AUDENC_CFP_CHANNEL_MONO;
else
    AUDCH = 2;
    AUDENC_NUMCH = Q_AUDENC_CFP_CHANNEL_STEREO;
end

board.setAudioInput({port=0, freq=SAMPLERATE, mode=AIN});

if (AIN ~= "kite") then
  board.setAudioInput({port=1, freq=SAMPLERATE, mode=AIN});
end

avoc0 = sysctl0:avocCreate({name="avoc0"});
ws = sysctl0:createWriteStreamer({name="ws"});

ws:start();

avsn0 = sysctl0:avsnCreate({name="avsn0"});

sain = {};
audenc = {};

for i = 0, 1 do
    sain[i] = sysctl0:sainCreate({name="sain"..i, port=i});
    
    sain[i]:setParam({param=Q_SAIN_CMP_AUD_CHANNEL_NUM, value=Q_SAIN_CFP_CHANNEL_STEREO});
    sain[i]:setParam({param=Q_SAIN_CMP_AUD_SERIAL_MODE, value=Q_SAIN_CFP_AUD_SERIAL_MODE_I2S});
    sain[i]:setParam({param=Q_SAIN_CMP_AUD_MASTER_CLOCK, value=Q_SAIN_CFP_AUD_MASTER_CLOCK_256FS});
    sain[i]:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
    sain[i]:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_SIZE, value=16});
    sain[i]:setParam({param=Q_SAIN_CMP_AUD_FRAME_SIZE, value=AUDIO_FRAME_SIZE});
    if (AIN == 'kite') then
        sain[i]:setParam({param=Q_SAIN_CMP_AUD_CLOCK_SOURCE, value=Q_SAIN_CFP_AUD_EXTERNAL_CLK});
    end
    sain[i]:activateCfg();
    
    if ((PREVIEW == 1) and (i == AUDIO_OUTPUT_SRC)) then
        avsn0:bindAudioInput({src=sain[i]});
        ach = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_STEREO});
        avoc0:setAudioChannelSource({ch=ach, src=avsn0});
        avoc0:enableAudioChannel({ch=ach, enable=1});
    end
end

if (SPLIT_STEREO_INPUT == 1) then
    asplit = {};
    for i = 0, 1 do
        asplit[i] = sysctl0:asplitCreate({name="asplit"..i, port=i, numOutput=2});
        asplit[i]:bindAudioInput({src=sain[i]});
    end
    
    if (PREVIEW == 2) then
        if (AUDIO_OUTPUT_SRC <= 1) then
            avsn0:bindAudioInput({src=asplit[0], port=(AUDIO_OUTPUT_SRC%2)});
        else
            avsn0:bindAudioInput({src=asplit[1], port=(AUDIO_OUTPUT_SRC%2)});
        end
        ach = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_MONO_L});
        avoc0:setAudioChannelSource({ch=ach, src=avsn0});
        avoc0:enableAudioChannel({ch=ach, enable=1});
    end
end

if (AUDIO_ENCODE_ON == 1) then
    for i = 0, AUDCH-1 do
        local audioStreamId = i + 1;
        
        audenc[i] = sysctl0:audencCreate({name="audenc"..i, ws=ws, codecType=AUDIO_CODEC});
        
        if (SPLIT_STEREO_INPUT == 0) then
            audenc[i]:bindAudioInput({src=sain[i]});
        else
            -- ch0: sain0 + ch0, ch1: sain0 + ch1
            -- ch2: sain1 + ch0, ch3: sain1 + ch1
            if (i <= 1) then
                audenc[i]:bindAudioInput({src=asplit[0], port=(i%2)});
            else
                audenc[i]:bindAudioInput({src=asplit[1], port=(i%2)});
            end
        end
        
        audenc[i]:setStreamID({streamid=audioStreamId});
        
        audenc[i]:setParam({param=Q_AUDENC_CMP_NUM_CHANNELS, value=AUDENC_NUMCH});
        audenc[i]:setParam({param=Q_AUDENC_CMP_Q711_LAW, value=Q_AUDENC_CFP_Q711_U_LAW});
        audenc[i]:setParam({param=Q_AUDENC_CMP_SAMPLE_RATE, value=SAMPLERATE});
        audenc[i]:setParam({param=Q_AUDENC_CMP_SAMPLE_SIZE, value=16});
        audenc[i]:setParam({param=Q_AUDENC_CMP_BIT_RATE, value=ABITRATE});
        audenc[i]:activateCfg();
        
        ws:open({type="file", file=OUTPUTFILENAME[i], sid = {audioStreamId}});
    end
end

avoc0:setParam({param=Q_AVOC_CMP_AUD_SERIAL_MODE, value=Q_AVOC_CFP_AUD_SERIAL_MODE_I2S});
avoc0:setParam({param=Q_AVOC_CMP_AUD_MASTER_CLOCK, value=Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_SIZE, value=16});
avoc0:activateCfg();

avoc0:activateAudioChannelCfg();

avoc0:start();
avsn0:start();

for i = 0, 1 do
    sain[i]:start();
end

if (AUDIO_ENCODE_ON == 1) then
    for i = 0, AUDCH-1 do
        audenc[i]:record();
    end
end


