--[[

This script is used to decode the input file specified. 

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Video output mode
-- "hdmi" HDMI output (1080i and 720p only)
-- "component" Component output
-- "svideo" SVIDEO output (note composite is driven at same time on output)
VOUT        	= VOUT or "hdmi";

-- Video format
-- "1080i"
-- "720p"
-- "480i"
-- "576i"
FORMAT      	= FORMAT or "1080i";

VIDEO_CODEC 	= VIDEO_CODEC or Q_AVDEC_CFP_VIDEO_CODEC_AVC;
--VIDEO_STREAM    = VIDEO_STREAM or Q_AVDEC_CFP_VIDEO_STREAM_TS; 
VIDEO_STREAM    = VIDEO_STREAM or Q_AVDEC_CFP_VIDEO_STREAM_QBOX; 

VIDEO_PID       = VIDEO_PID or 4113; 

-- Audio Codec (cannot be specified with -e option as the value passed will always be 0; change here or use another configuration Lua script)
-- Q_AVDEC_CFP_AUDIO_CODEC_QAC
-- Q_AVDEC_CFP_AUDIO_CODEC_Q711
-- Q_AVDEC_CFP_AUDIO_CODEC_Q722
-- Q_AVDEC_CFP_AUDIO_CODEC_Q726
-- Q_AVDEC_CFP_AUDIO_CODEC_QMA
-- Q_AVDEC_CFP_AUDIO_CODEC_QPCM
AUDIO_CODEC = AUDIO_CODEC or Q_AVDEC_CFP_AUDIO_CODEC_QAC;
AUDIO_PID       = AUDIO_PID or 4114; -- 4114 aac; 4115 pcm

-- Used only for TS streams; can keep 0 for QBOX
FASTPASSTHROUGH = 0;

-- Audio sample rate and stereo/mono (depending on codec)
if (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_QAC) then
  SAMPLERATE  = SAMPLERATE or 32000;
  AUDIO_STEREO = AUDIO_STEREO or 1;
  if (VIDEO_STREAM == Q_AVDEC_CFP_VIDEO_STREAM_TS) then
    FASTPASSTHROUGH = 1;
  end
elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_Q711) or (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_Q726) then
  SAMPLERATE  = 8000;
  AUDIO_STEREO = 0;
elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_Q722) then
  SAMPLERATE = 16000;
  AUDIO_STEREO = 1;
elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_QMA) then
  SAMPLERATE  = SAMPLERATE or 32000;
  AUDIO_STEREO = 1;
elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_QPCM) then
  SAMPLERATE  = SAMPLERATE or 32000;
  AUDIO_STEREO = 1;
else 
  print("Unknown audio codec type selected\n");
end

-- 0 for none, 1 for UDP, 2 for RTP
STREAMING = STREAMING or 0;
-- Port number used in UDP
PORT = PORT or 9700;

-- IP Address of server for RTP
SERVER_IP = SERVER_IP or "10.32.84.176";

-- Server port used in RTP
SERVER_PORT = SERVER_PORT or 554;

-- session identifier for rtp server
SESSION_ID = SESSION_ID or "test";

-- RTP Data protocol - "udp" or "tcp"
RTP_DATA_PROTO = RTP_DATA_PROTO or "udp";

-- Client Port number
CLIENT_PORT = CLIENT_PORT or 1024;

-- Maximum number of retries to connect to RTSP Server
-- 0 - Try connecting to server infinitely
-- n - Try connecting to server n times, 
--     if fails connecting to server after n times,
--     rtp bitstream reader will be closed      
MAX_CONNECT_RETRY = MAX_CONNECT_RETRY or 0;

-- Set Target Latency Values
--TARGET_LATENCY_TOLERANCE = TARGET_LATENCY_TOLERANCE or 150; -- in milliseconds
--TARGET_LATENCY_UNDERFLOW = TARGET_LATENCY_UNDERFLOW or 10;  -- in milliseconds
--TARGET_LATENCY_OVERFLOW  = TARGET_LATENCY_OVERFLOW or 290;   -- in milliseconds

-- Don't set Target Latency Values to test bypassing logic
TARGET_LATENCY_TOLERANCE = TARGET_LATENCY_TOLERANCE or 0; -- in milliseconds
TARGET_LATENCY_UNDERFLOW = TARGET_LATENCY_UNDERFLOW or 0;  -- in milliseconds
TARGET_LATENCY_OVERFLOW  = TARGET_LATENCY_OVERFLOW or 0;   -- in milliseconds

-- Set 1 to start client as third party client
START_AS_THIRD_PARTY_CLIENT = START_AS_THIRD_PARTY_CLIENT or 0;

if (START_AS_THIRD_PARTY_CLIENT == 1) then
	-- Set number of channels
	if (AUDIO_STEREO == 1) then
	 	NO_OF_CHANNELS = NO_OF_CHANNELS or 2;
	else
		NO_OF_CHANNELS = NO_OF_CHANNELS or 1;
	end

	-- Audio Codec name
	if (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_QAC) then
		AUDIO_CODEC_NAME = AUDIO_CODEC_NAME or "aac";
	elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_Q711)then
	-- Make is same as g711 A-Law or g711 U-Law as used on server side
	  AUDIO_CODEC_NAME = AUDIO_CODEC_NAME or "g711u";
	elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_Q726)then
		AUDIO_CODEC_NAME = AUDIO_CODEC_NAME or "g726";
		ABITRATE = ABITRATE  or 40000;
	elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_QPCM) then
		AUDIO_CODEC_NAME = AUDIO_CODEC_NAME or "pcm";
	elseif (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_QMA) then
		AUDIO_CODEC_NAME = AUDIO_CODEC_NAME or "mpa";
	end

	-- Video Client Port number
	TP_VIDEO_CLIENT_PORT = TP_VIDEO_CLIENT_PORT or 2048;

	-- Audio Client Port number
	TP_AUDIO_CLIENT_PORT = TP_VIDEO_CLIENT_PORT + 1;
end

-- UDP buffer size
UDP_BUFFERSIZE = UDP_BUFFERSIZE or 2000000;

INPUTFILENAME 	= INPUTFILENAME or "/directmnt/qa/streams/merlin/spider.1080i.qbox";

-- Use different streams for different formats
-- av480i.qbx is to be used if format specified is 480i
--av1080i.qbx is to be used if format specified is 1080i

--INPUTFILENAME = INPUTFILENAME or "/directmnt/qa/streams/merlin/spider.480i.qbox";
--INPUTFILENAME = INPUTFILENAME or "/directmnt/qa/streams/merlin/spider.720p.qbox";
--INPUTFILENAME = "test.qbx";
--INPUTFILENAME  		= INPUTFILENAME or "/directmnt/qa/streams/merlin/test_720p_IP.ts";
--INPUTFILENAME = "/directmnt/qa/streams/merlin/av_pcm.ts"; --16000 samplerate
--INPUTFILENAME = "/directmnt/qa/streams/merlin/test_480i_aac.ts"; --16000 samplerate

-- Allow video only decode
VIDEO_ONLY = VIDEO_ONLY or 0
AUDIO_ONLY = AUDIO_ONLY or 0

if (VIDEO_ONLY ~= 0) then
    AUDIO_PID = 0;
end

if (AUDIO_ONLY ~= 0) then
    VIDEO_PID = 0;
end

-- Direct memory transfer mode (aka AVC instant display mode)
MEMORY_XFER_MODE = MEMORY_XFER_MODE or 0;

--[[ Low delay decoder options
  0 = disable low delay mode.
  1 = Low Delay mode, works with IPPP only bit streams.
  2 = Instant Display mode, works with IPPP streams only.  Note that, this mode can not be used with multiple decoders.
--]]
LOW_DELAY_MODE = LOW_DELAY_MODE or 0;

--Force low delay mode for streaming
if ((STREAMING > 0) and (LOW_DELAY_MODE == 0)) then
  LOW_DELAY_MODE = 1;
end

-- Options for NTP sync mode for AVSN:
-- Enables NTP sync
NTP_SYNC_ENABLE = NTP_SYNC_ENABLE or 0;
-- Decoding an encoded AV file with NTP timestamps requires the application to supply a valid offset value to AVSN.
-- For example, if an AV file was recorded 4 hours ago and you want to playback the content of the file 5 seconds
-- from now, you have to supply a (4 * 3600) + 5 seconds of offset.
-- The second and microsecond offset values are added to a total value for NTP sync.
NTP_SYNC_OFFSET_SEC = NTP_SYNC_OFFSET_SEC or 0;   -- offset in seconds
NTP_SYNC_OFFSET_USEC = NTP_SYNC_OFFSET_USEC or 0;   -- offset in micro seconds
-- Lets the internal codes to calculate the correct offset to playback the encoded stream
NTP_SYNC_AUTO = NTP_SYNC_AUTO or 1;
-- The AUTO option has 3 different modes:
-- Q_AVSN_CFP_NTP_AUTO_MODE_INSTANT: Instant display of the audio/video bitstream.
-- Q_AVSN_CFP_NTP_AUTO_MODE_DELAY: Display of bitstream is delayed by a time defined by the offset values.
-- Q_AVSN_CFP_NTP_AUTO_MODE_SKIP: Initial part of bitstream is skipped by an amount defined by the offset values.
NTP_SYNC_AUTO_MODE = NTP_SYNC_AUTO_MODE or Q_AVSN_CFP_NTP_AUTO_MODE_INSTANT;
-- The input audio frames with NTP timestamps may come to the AVSN at a rate different from the rate
-- of the audio hardware interrupts. Therefore, if no corrections are being done, the audio and video
-- may become out-of-sync. The follow parameter will enable the use of an audio filter to
-- resample the input audio frames to correct the difference in rates.
NTP_SYNC_AUDIO_RESAMPLE_ENABLE = NTP_SYNC_AUDIO_RESAMPLE_ENABLE or 1;
-- This parameter only matters when audio resample is enabled.
-- At each audio interrupt, the current NTP time (STC) is read from the NTP clock. The PTS of the next audio frame 
-- ready to play is also noted. The parameter will tell the audio filter to maintain a target divergence
-- between the STC and PTS values. A positive value means the PTS will lead the STC (PTS > STC) and a negative
-- value means the PTS will lag behind the STC (PTS < STC).
-- The units of this paramter is number of ticks of the system clock.
NTP_SYNC_AUDIO_RESAMPLE_TARGET_DIVERGENCE = NTP_SYNC_AUDIO_RESAMPLE_TARGET_DIVERGENCE or 1000;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------
if (LOW_DELAY_MODE == 2) then
  if (MEMORY_XFER_MODE ~= 1) then
    print("Warning, Instant Display Mode is on, decoder is now forced to use DMA mode");
    MEMORY_XFER_MODE = 1;
  end
end

if (MEMORY_XFER_MODE == 1) then
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC_INSTANT_DISPLAY;
else
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC;
end

-- Allow the display to be disabled for power testing
DISABLE_DISPLAY = DISABLE_DISPLAY or 0

SHOWUPDATE = SHOWUPDATE or 0;

PTS_120KHZ = PTS_120KHZ or 0;

if (FORMAT == "720p") then
  HSIZE = 1280;
  VSIZE = 720;
  VIDEO_FRAME_DURATION = 1000000 / 60;
  CLOCK_NTSC = CLOCK_NTSC or 0;
  if (PTS_120KHZ == 1) then
    VIDEO_FRAME_TIME = 2002;
  else
    VIDEO_FRAME_TIME = 1500;
  end
elseif (FORMAT == "1080i") then
  HSIZE = 1920;
  VSIZE = 1080;
  VIDEO_FRAME_DURATION = 1000000 / 30;
  CLOCK_NTSC = CLOCK_NTSC or 1;
  if (PTS_120KHZ == 1) then
    VIDEO_FRAME_TIME = 4004;
  else
    VIDEO_FRAME_TIME = 3003;
  end
elseif (FORMAT == "480i") then
  HSIZE = 720;
  VSIZE = 480;
  VIDEO_FRAME_DURATION = 1000000 / 30;
  CLOCK_NTSC = 0;
  if (PTS_120KHZ == 1) then
    VIDEO_FRAME_TIME = 4004;
  else
    VIDEO_FRAME_TIME = 3003;
  end
elseif (FORMAT == "576i") then
  HSIZE = 720;  
  VSIZE = 576;
  VIDEO_FRAME_DURATION = 1000000 / 25;
  CLOCK_NTSC = 0;
  if (PTS_120KHZ == 1) then
    VIDEO_FRAME_TIME = 4800;
  else
    VIDEO_FRAME_TIME = 3600;
  end
else
  print("Invalid format\n");
end

AUDIO_BUFFER_SIZE = 1000000;
VIDEO_BUFFER_SIZE = 2000000;
VIDEO_STREAM_ID = 2;
AUDIO_STREAM_ID = 1;


function play()
  if (STREAMING == 0) then
    if (VIDEO_STREAM == Q_AVDEC_CFP_VIDEO_STREAM_TS) then
      bsr0 = rs0:open({type="dump", file=INPUTFILENAME, srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
    else
      bsr0 = rs0:open({type="qbox", file=INPUTFILENAME, srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
    end
  elseif (STREAMING == 1) then
    bsr0 = rs0:open({type="udp", file=string.format("port=%s,bufferlen=%s", PORT, UDP_BUFFERSIZE),
      srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
  elseif (STREAMING == 2) then
    if (START_AS_THIRD_PARTY_CLIENT == 1) then
      if (VIDEO_STREAM == Q_AVDEC_CFP_VIDEO_STREAM_TS) then
        bsr0 = rs0:open({type="rtp", file=string.format("sat=1,tvc=%s,tvp=%d,tlt=%d,tlo=%d,tlu=%d",
               "mp2t", TP_VIDEO_CLIENT_PORT,
                TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW),
                srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
      elseif (VIDEO_ONLY == 0) then
        if (AUDIO_CODEC_NAME == "g726") then
          bsr0 = rs0:open({type="rtp", file=string.format("sat=1,tvc=%s,tac=%s,tvp=%d,tap=%d,tcr=%d,tnc=%d,tbr=%d,tlt=%d,tlo=%d,tlu=%d",
                   "h264", AUDIO_CODEC_NAME ,TP_VIDEO_CLIENT_PORT, TP_AUDIO_CLIENT_PORT, SAMPLERATE, NO_OF_CHANNELS,ABITRATE,
                    TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW),
                    srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
        else
          bsr0 = rs0:open({type="rtp", file=string.format("sat=1,tvc=%s,tac=%s,tvp=%d,tap=%d,tcr=%d,tnc=%d,tlt=%d,tlo=%d,tlu=%d",
            "h264", AUDIO_CODEC_NAME ,TP_VIDEO_CLIENT_PORT, TP_AUDIO_CLIENT_PORT, SAMPLERATE, NO_OF_CHANNELS,
            TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW),
            srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
        end
      else
        bsr0 = rs0:open({type="rtp", file=string.format("sat=1,tvc=%s,tvp=%d,tlt=%d,tlo=%d,tlu=%d",
               "h264", TP_VIDEO_CLIENT_PORT,
                TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW),
                srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
      end
    else
      -- Not a third party client
      bsr0 = rs0:open({type="rtp", file=string.format("sip=%s,prt=%d,dat=%s,ses=%s,tlt=%d,tlo=%d,tlu=%d,cpt=%d,mcr=%d",
                     SERVER_IP, SERVER_PORT, RTP_DATA_PROTO, SESSION_ID,
                     TARGET_LATENCY_TOLERANCE, TARGET_LATENCY_OVERFLOW, TARGET_LATENCY_UNDERFLOW, CLIENT_PORT, MAX_CONNECT_RETRY),
                     srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
    end
  end

  if ((VIDEO_ONLY == 0) and (STREAMING > 0)) then
    audfil0:start();
  end

  if ( (VIDEO_ONLY == 0) and (NTP_SYNC_ENABLE == 1) and (NTP_SYNC_AUDIO_RESAMPLE_ENABLE == 1) ) then
    audfil0:start();
  end

  rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});

  avsn0:start();
  avdec0:start();
  rs0:play({bsr=bsr0});
end

function stop()
  rs0:close({bsr=bsr0});
  avdec0:stop();
  avdec0:waitForVideoEOS();
  avsn0:stop();
end

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

-- Set up the output port
--if (DISABLE_DISPLAY == 0) then
  if (FORMAT == "1080i") then
    board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, clockntsc=CLOCK_NTSC, sync=vout_sync});
  else
    board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, clockntsc=CLOCK_NTSC});
  end
--end

if (VIDEO_ONLY == 0) then
  if (SAMPLERATE == 8000) then
    board.setAudioSampleFreq({port=0, freq=16000});
    sysctl0:configure({param=Q_SYS_CFG_PLL3_CLOCK_HZ, value=2048000});
    sysctl0:configure({param=Q_SYS_CFG_AUD0_CLOCK_HZ, value=2048000});
  else
    board.setAudioSampleFreq({port=0, freq=SAMPLERATE});
  end
end

-- create objects
avoc0 = sysctl0:avocCreate({name="avoc0"});
avsn0 = sysctl0:avsnCreate({name="avsn0"});
avdec0 = sysctl0:avdecCreate({name="avdec0", atype=AUDIO_CODEC, vtype=VIDEO_CODEC, vstream=VIDEO_STREAM, audioBufferSize=AUDIO_BUFFER_SIZE,
                              videoBufferSize=VIDEO_BUFFER_SIZE, maxWidth=HSIZE, maxHeight=VSIZE, ioMode=MEMORY_XFER_MODE, fastPassThrough=FASTPASSTHROUGH});
rs0 = sysctl0:createReadStreamer({name="rs0"});

if (PTS_120KHZ == 1) then
  sysctl0:configure({param=Q_SYS_CFG_PTS_PER_SECOND, value=120000});
end

avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=AUDIO_STREAM_ID});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=VIDEO_STREAM_ID});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=SAMPLERATE});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});
if (VIDEO_STREAM == Q_AVDEC_CFP_VIDEO_STREAM_TS) then
    avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_TYPE, value=Q_AVDEC_CFP_AUDIO_STREAM_TS});
    avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TS_PID, value=VIDEO_PID});
    avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_TS_PID, value=AUDIO_PID});
    avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_OVERRIDE, value=Q_AVDEC_CFP_TRUE});
    avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_DEFAULT, value=VIDEO_FRAME_TIME});
else
    avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_TYPE, value=Q_AVDEC_CFP_AUDIO_STREAM_QBOX});
end

if (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_QAC) then 
    avdec0:setParam({param=Q_AVDEC_CMP_AAC_OBJECT_TYPE, value=Q_AVDEC_CFP_AAC_LC});
end

if (AUDIO_STEREO == 1) then
  avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_STEREO});
else
  avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_MONO});
end
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_INITIAL_BUFFER_LEVEL, value=0});

if (LOW_DELAY_MODE == 1) then
  avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_OUTPUT_TIMING, value=Q_AVDEC_CFP_VIDEO_OUTPUT_TIMING_LOW_DELAY});
elseif (LOW_DELAY_MODE == 2) then
  avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_SLICE_DECODE_MODE, value=1});
  avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_OUTPUT_TIMING, value=Q_AVDEC_CFP_VIDEO_OUTPUT_TIMING_ZERO_DELAY});
else
  -- normal output timing mode
end

avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_INITIAL_BUFFER_LEVEL, value=0});

avdec0:activateCfg();

-- set video hw params
if (FORMAT == "480i") then
  avoc0:setOutput480i();
elseif (FORMAT == "576i") then
  avoc0:setOutput576i();
elseif (FORMAT == "720p") then
  avoc0:setOutput720p60();
elseif (FORMAT == "1080i") then
  avoc0:setOutput1080i60({sync=vout_sync});
end

avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=HSIZE});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=VSIZE});

avoc0:setParam({param=Q_AVOC_CMP_VID_OUTPUT_TICKS, value=VIDEO_FRAME_TIME});

avoc0:setParam({param=Q_AVOC_CMP_AUD_SERIAL_MODE, value=Q_AVOC_CFP_AUD_SERIAL_MODE_I2S});
avoc0:setParam({param=Q_AVOC_CMP_AUD_MASTER_CLOCK, value=Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_SIZE, value=16});
if (DISABLE_DISPLAY ~= 0) then
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_ENABLE, value=0});
end
avoc0:activateCfg();

avsn0:bindVideoInput({src=avdec0});

if (STREAMING > 0) then
  -- We modified "SO_RCVBUF" size to 1024*1024 using following Linux commands so that
  -- we can receive the stream without any packet drops:
  os.execute("sysctl -w net.core.rmem_default=1048576");  -- (default was 104448)
  os.execute("sysctl -w net.core.rmem_max=1048576");      -- (default was 104448)
  -- Uncomment following code to reduce board to board latency in video only mode
  -- though it may result in occassional frame drops
  avsn0:setParam({param=Q_AVSN_CMP_VIDEO_STARTUP_LATENCY, value=math.ceil(75000 / VIDEO_FRAME_DURATION)});
  avsn0:setParam({param=Q_AVSN_CMP_VID_FIXED_LATENCY_ENABLE, value=1});
  avsn0:setParam({param=Q_AVSN_CMP_VID_FIXED_LATENCY_TARGET, value=75000});
  avsn0:setParam({param=Q_AVSN_CMP_VID_FIXED_LATENCY_THRESHOLD, value=math.ceil(VIDEO_FRAME_DURATION / 1.5)});
  avsn0:setParam({param=Q_AVSN_CMP_MAX_QUEUED_VIDEO_FRAMES, value=math.ceil(133000 / VIDEO_FRAME_DURATION)});
  avsn0:activateCfg();
end

if (STREAMING == 2) then
  -- Following lines were added to avoid a deadlock in AVSN when it only receives
  -- audio packets in beginning. This can happen depending on where in the GOP we
  -- are when starting the streaming. The video decoder has to wait for the next GOP
  -- if we are not in the beginning of GOP when streaming is started.
  -- However, this option is not fully tested and is causing problems like
  -- jerky playback on some use cases like 480i 2Mb/sec. Therefore, it is kept
  -- commented. If you observe hang in beginning, then uncomment these lines.
  -- avsn0:setParam({param=Q_AVSN_CMP_STARTUP_SYNCH_DROP_FRAMES, value=1});
  -- avsn0:activateCfg();
end

if (VIDEO_ONLY == 0) then
  if (STREAMING > 0) then
    -- Create an audio filter for resampling, and bind it between the decoder and the AVSN
    audfil0 = sysctl0:audfilterCreate({name="audfil0",numOutputPorts=1,numOutputFrames=16,maxFrameSize=12288,filterType=Q_AUDFLTR_CFP_TYPE_SLIPPAGECONTROL,cBufferLen=8});

    audfil0:setParam({param=Q_AUDFLTR_CMP_MAX_INPUT_PORTS, value=1});
    audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_NUM_CHANNELS, value=1});
    audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_IN_FRAME_NUM_SAMPLES, value=256});

    audfil0:activateCfg();

    afinport = audfil0:addInputPort();
    audfil0:setInputPortSource({inport=afinport,src=avdec0,threshold=1});

    -- Control the latency at playback start
    avsn0:setParam({param=Q_AVSN_CMP_AUDIO_STARTUP_LATENCY, value=5});
    -- Control the long term latency (in PTS)
    avsn0:setParam({param=Q_AVSN_CMP_AUD_LATENCY_RESAMPLE_ENABLE, value=1});
    avsn0:setParam({param=Q_AVSN_CMP_AUD_LATENCY_TARGET, value=75000});
    avsn0:setParam({param=Q_AVSN_CMP_MAX_QUEUED_AUDIO_FRAMES, value=8});
    avsn0:activateCfg();

    avsn0:bindAudioInput({src=audfil0, port=0});
  elseif ( (NTP_SYNC_ENABLE == 1) and (NTP_SYNC_AUDIO_RESAMPLE_ENABLE == 1) ) then
    audfil0 = sysctl0:audfilterCreate({name="audfil0",numOutputPorts=1,numOutputFrames=16,maxFrameSize=12288,filterType=Q_AUDFLTR_CFP_TYPE_SLIPPAGECONTROL});
    audfil0:setParam({param=Q_AUDFLTR_CMP_MAX_INPUT_PORTS, value=1});
    audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_NUM_CHANNELS, value=1});
    audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_IN_FRAME_NUM_SAMPLES, value=256});

    audfil0:activateCfg();
    afinport = audfil0:addInputPort();
    audfil0:setInputPortSource({inport=afinport,src=avdec0,threshold=1});

    avsn0:bindAudioInput({src=audfil0, port=0});
  else
    avsn0:bindAudioInput({src=avdec0});
  end

  if (AUDIO_STEREO == 1) then
    ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_STEREO});
  else
    ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_MONO_L});
  end
  avoc0:setAudioChannelSource({ch=ach0, src=avsn0});
  avoc0:enableAudioChannel({ch=ach0, enable=1});
  avoc0:activateAudioChannelCfg();
else
  AUDIO_STREAM_ID = nil; 
end

vch0 = avoc0:addVideoChannel();
avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=HSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=VSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=HSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=VSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});

avoc0:activateVideoChannelCfg();

if (NTP_SYNC_ENABLE == 1) then
   -- needs to disable the PTS threshold as it will interfere with the NTP sync mode for AVSN
   avsn0:setParam({param=Q_AVSN_CMP_PTS_RESET_THRESHOLD, value=0});

   avsn0:setParam({param=Q_AVSN_CMP_NTP_DIVERGENCE_RESAMPLE_ENABLE,      value=NTP_SYNC_AUDIO_RESAMPLE_ENABLE});
   avsn0:setParam({param=Q_AVSN_CMP_NTP_DIVERGENCE_TARGET,               value=NTP_SYNC_AUDIO_RESAMPLE_TARGET_DIVERGENCE});

   -- The following settins are found to be fairly good at converging the divergence to the target value
   avsn0:setParam({param=Q_AVSN_CMP_NTP_DIVERGENCE_CHECK_INTERVAL,       value=10});
   avsn0:setParam({param=Q_AVSN_CMP_NTP_DIVERGENCE_RESAMPLE_INCREMENT,   value=1});
   avsn0:setParam({param=Q_AVSN_CMP_NTP_DIVERGENCE_RESAMPLE_MAX,         value=1024});
   avsn0:setParam({param=Q_AVSN_CMP_NTP_DIVERGENCE_THRESHOLD_0,          value=10});
   avsn0:setParam({param=Q_AVSN_CMP_NTP_DIVERGENCE_THRESHOLD_1,          value=20});
   avsn0:setParam({param=Q_AVSN_CMP_NTP_DIVERGENCE_THRESHOLD_2,          value=60});
   avsn0:setParam({param=Q_AVSN_CMP_NTP_DIVERGENCE_THRESHOLD_RESAMPLE_1, value=1024});
   avsn0:setParam({param=Q_AVSN_CMP_NTP_DIVERGENCE_THRESHOLD_RESAMPLE_2, value=2048});

   avsn0:activateCfg();

   avsn0:enableNtpSync({enable=1, auto=NTP_SYNC_AUTO, autoMode=NTP_SYNC_AUTO_MODE, offsetSec=NTP_SYNC_OFFSET_SEC, offsetUSec=NTP_SYNC_OFFSET_USEC});
end

rs0:bindDecoder({dec=avdec0});
rs0:setParam({param=Q_RS_CFG_SHOW_UPDATE, value=SHOWUPDATE});

avoc0:start();
play();

