
--[[

This script is used to decode 4 streams simultaneously 
and display on the screen. 

--]]
----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Number of decoded channels
DECCH       = DECCH or 4;

-- Number of rendered images and their scaled size
OUTCH       = OUTCH or 4;
FORMAT_OUT  = FORMAT_OUT or "1080i";
FORMAT_IN   = FORMAT_IN or "480i";
VIN         = VIN or "svideo";
VOUT        = VOUT or "component";
OUTW        = OUTW or 720;
OUTH        = OUTH or 480;

-- Width and height of the output grid
OUTGRIDW    = OUTGRIDW or 2;
OUTGRIDH    = OUTGRIDH or 2;

STREAMING   = STREAMING or 0;

if (STREAMING == 0) then
  -- Input file names (# of files should match # of decoded channels)
  INPUTFILENAME    = INPUTFILENAME or {};
  INPUTFILENAME[0] = INPUTFILENAME[0] or "/directmnt/qa/streams/merlin/movie1.480i.qbox";
  INPUTFILENAME[1] = INPUTFILENAME[1] or "/directmnt/qa/streams/merlin/movie2.480i.qbox";
  INPUTFILENAME[2] = INPUTFILENAME[2] or "/directmnt/qa/streams/merlin/movie3.480i.qbox";
  INPUTFILENAME[3] = INPUTFILENAME[3] or "/directmnt/qa/streams/merlin/movie4.480i.qbox";
else
  -- IP Address of server for RTP
  SERVER_IP        = SERVER_IP or {};
  SERVER_IP[0]     = SERVER_IP[0] or "10.32.84.176";
  SERVER_IP[1]     = SERVER_IP[1] or "10.32.84.176";
  SERVER_IP[2]     = SERVER_IP[2] or "10.32.84.176";
  SERVER_IP[3]     = SERVER_IP[3] or "10.32.84.176";

  -- Server port used in RTP
  SERVER_PORT      = SERVER_PORT or {};
  SERVER_PORT[0]   = SERVER_PORT[0] or 554;
  SERVER_PORT[1]   = SERVER_PORT[1] or 554;
  SERVER_PORT[2]   = SERVER_PORT[2] or 554;
  SERVER_PORT[3]   = SERVER_PORT[3] or 554;

  -- session identifier for rtp server
  SESSION_ID       = SESSION_ID or {};
  SESSION_ID[0]    = SESSION_ID[0] or "test";
  SESSION_ID[1]    = SESSION_ID[1] or "test";
  SESSION_ID[2]    = SESSION_ID[2] or "test";
  SESSION_ID[3]    = SESSION_ID[3] or "test";

  -- RTP Data protocol - "udp" or "tcp"
  RTP_DATA_PROTO    = RTP_DATA_PROTO or {};
  RTP_DATA_PROTO[0] = RTP_DATA_PROTO[0] or "udp";
  RTP_DATA_PROTO[1] = RTP_DATA_PROTO[1] or "udp";
  RTP_DATA_PROTO[2] = RTP_DATA_PROTO[2] or "udp";
  RTP_DATA_PROTO[3] = RTP_DATA_PROTO[3] or "udp";

  -- Don't set Target Latency Values to test bypassing logic
  TARGET_LATENCY_TOLERANCE    = TARGET_LATENCY_TOLERANCE or {};
  TARGET_LATENCY_TOLERANCE[0] = TARGET_LATENCY_TOLERANCE[0] or 90; -- in milliseconds
  TARGET_LATENCY_TOLERANCE[1] = TARGET_LATENCY_TOLERANCE[1] or 90; -- in milliseconds
  TARGET_LATENCY_TOLERANCE[2] = TARGET_LATENCY_TOLERANCE[2] or 90; -- in milliseconds
  TARGET_LATENCY_TOLERANCE[3] = TARGET_LATENCY_TOLERANCE[3] or 90; -- in milliseconds
  TARGET_LATENCY_UNDERFLOW    = TARGET_LATENCY_UNDERFLOW or {};
  TARGET_LATENCY_UNDERFLOW[0] = TARGET_LATENCY_UNDERFLOW[0] or 0;  -- in milliseconds
  TARGET_LATENCY_UNDERFLOW[1] = TARGET_LATENCY_UNDERFLOW[1] or 0;  -- in milliseconds
  TARGET_LATENCY_UNDERFLOW[2] = TARGET_LATENCY_UNDERFLOW[2] or 0;  -- in milliseconds
  TARGET_LATENCY_UNDERFLOW[3] = TARGET_LATENCY_UNDERFLOW[3] or 0;  -- in milliseconds
  TARGET_LATENCY_OVERFLOW     = TARGET_LATENCY_OVERFLOW or {};
  TARGET_LATENCY_OVERFLOW[0]  = TARGET_LATENCY_OVERFLOW[0] or 180;   -- in milliseconds
  TARGET_LATENCY_OVERFLOW[1]  = TARGET_LATENCY_OVERFLOW[1] or 180;   -- in milliseconds
  TARGET_LATENCY_OVERFLOW[2]  = TARGET_LATENCY_OVERFLOW[2] or 180;   -- in milliseconds
  TARGET_LATENCY_OVERFLOW[3]  = TARGET_LATENCY_OVERFLOW[3] or 180;   -- in milliseconds

  -- Audio ring buffer size
  AUDIO_RING_BUFF_SIZE        = AUDIO_RING_BUFF_SIZE or {};
  AUDIO_RING_BUFF_SIZE[0]     = AUDIO_RING_BUFF_SIZE[0] or 524288;
  AUDIO_RING_BUFF_SIZE[1]     = AUDIO_RING_BUFF_SIZE[1] or 524288;
  AUDIO_RING_BUFF_SIZE[2]     = AUDIO_RING_BUFF_SIZE[2] or 524288;
  AUDIO_RING_BUFF_SIZE[3]     = AUDIO_RING_BUFF_SIZE[3] or 524288;

  -- Video ring buffer size
  VIDEO_RING_BUFF_SIZE        = VIDEO_RING_BUFF_SIZE or {};
  VIDEO_RING_BUFF_SIZE[0]     = VIDEO_RING_BUFF_SIZE[0] or 1572864;
  VIDEO_RING_BUFF_SIZE[1]     = VIDEO_RING_BUFF_SIZE[1] or 1572864;
  VIDEO_RING_BUFF_SIZE[2]     = VIDEO_RING_BUFF_SIZE[2] or 1572864;
  VIDEO_RING_BUFF_SIZE[3]     = VIDEO_RING_BUFF_SIZE[3] or 1572864;
end


-- Direct memory transfer mode (aka AVC instant display mode)
MEMORY_XFER_MODE = MEMORY_XFER_MODE or 1;

-- Play repeatly 
REPEAT_MODE = REPEAT_MODE or 1;

--[[ Low delay decoder options
  0 = disable low delay mode.
  1 = Low Delay mode, works with IPPP only bit streams.
--]]
LOW_DELAY_MODE = LOW_DELAY_MODE or 0;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

if (MEMORY_XFER_MODE == 1) then
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC_INSTANT_DISPLAY;
else
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC;
end

videoInputBufferSize = 500 * 2048;
audioInputBufferSize = 50 * 2048;

BITSTREAM_OUTPUT = BITSTREAM_OUTPUT or 1;

sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});
sysctl0:setVPortMode({port=1, live=0});
sysctl0:configure({param=Q_SYS_CFG_DECODER_INPUT_PARTITION_SIZE, value=DECCH*videoInputBufferSize+DECCH*audioInputBufferSize});

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

board.setVideoOutput({port=0, format=FORMAT_OUT, mode=VOUT, sync=vout_sync});
board.setVideoInput({port=0, format=FORMAT_IN, mode=VIN});

sysctl0:setVerboseLevel({param=1});

COMPWIDTH=OUTGRIDW*OUTW;
COMPHEIGHT=OUTGRIDH*OUTH;
COMPYOFFSET= (1080-(OUTGRIDH*OUTH))/2;
COMPXOFFSET= (1920-(OUTGRIDW*OUTW))/2;

-- show or hide bitrate 0 for off, 1 for on
SHOWRATE = 0;

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
avoc0:setOutput1080i60({sync=vout_sync});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=COMPXOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000});
avoc0:activateCfg();

if (STREAMING > 0) then
  -- Restore the socket receive buffer kernel parameters to default
  -- This is needed as they might be increased if avdecode.lua was executed earlier
  os.execute("sysctl -w net.core.rmem_default=104448");
  os.execute("sysctl -w net.core.rmem_max=104448");
end

-- Create Decoders
avdec = {};
avsndec = {};
DECCH = DECCH - 1;

for i = 0,DECCH do
    local audioStreamId = 2*i + 1;
    local videoStreamId = 2*i + 2;

    avdec[i] = sysctl0:avdecCreate({name="avdec", atype=Q_AVDEC_CFP_AUDIO_CODEC_QMA, vtype=VIDEO_CODEC, audioBufferSize=audioInputBufferSize, videoBufferSize=videoInputBufferSize, maxWidth=720, maxHeight=480, numoutputframes=8, videoAUs=200});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=audioStreamId});
    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=videoStreamId});
    --avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_DISABLE_OUTPUT, value=Q_AVDEC_CFP_TRUE});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=24000});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_STEREO});
    if(REPEAT_MODE == 1) then
        avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_OVERRIDE, value=1});
        avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_DEFAULT, value=3000});
    end
    avdec[i]:activateCfg();
end

vch = {};
x = 0;
y = 0;
OUTCH = OUTCH - 1;

for i = 0,OUTCH do
    avsndec[i] = sysctl0:avsnCreate({name="avsndec"});
    
    vch[i] = avoc0:addVideoChannel();
    
    avsndec[i]:bindVideoInput({src=avdec[i]});

    avoc0:setVideoChannelSource({ch=vch[i], src=avsndec[i]});
    
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=OUTW});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=OUTH});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});
    
    x = x + OUTW;
    if x == COMPWIDTH then
        x = 0;
        y = y + OUTH;
    end
end

avoc0:activateVideoChannelCfg();

bsr = {};
rs0 = sysctl0:createReadStreamer({name="rs0"});


function q()
    for i = 0,DECCH do
        avdec[i]:query();
    end
end

function DecStart(i)
    local audioStreamId = 2*i + 1;
    local videoStreamId = 2*i + 2;
    
    rs0:bindDecoder({dec=avdec[i]});
    if (STREAMING == 0) then
      bsr[i] = rs0:open({type="qbox", file=INPUTFILENAME[i], srcId={1, 2}, codecId={audioStreamId, videoStreamId}});
    else
      bsr[i] = rs0:open({type="rtp", file=string.format("sip=%s,prt=%d,dat=%s,ses=%s,tlt=%d,tlo=%d,tlu=%d,vbs=%d,abs=%d",
                SERVER_IP[i], SERVER_PORT[i], RTP_DATA_PROTO[i], SESSION_ID[i],
                TARGET_LATENCY_TOLERANCE[i], TARGET_LATENCY_OVERFLOW[i], TARGET_LATENCY_UNDERFLOW[i],
                VIDEO_RING_BUFF_SIZE[i], AUDIO_RING_BUFF_SIZE[i]),
                srcId={videoStreamId, audioStreamId}});
    end
    rs0:setReaderParam({bsr=bsr[i], param=Q_RS_CFG_BSR_LOOPMODE, value=REPEAT_MODE});
    rs0:setReaderParam({bsr=bsr[i], param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
    avsndec[i]:start();
    avdec[i]:start();
    rs0:play({bsr=bsr[i]});
end


function DecStop(i)
    rs0:close({bsr=bsr[i]});
    avdec[i]:stop();
    avdec[i]:waitForVideoEOS();
    avsndec[i]:stop();
    rs0:unbindDecoder({dec=avdec[i]});
end

function StartStopCh(Ch, time)
    os.execute("sleep "..time);
    DecStop(Ch);
    DecStart(Ch);
end

function StartStopTest()
    while 1 do
        for i = 0, DECCH do
            StartStopCh(i, 7);
        end
    end
end



-- START
avoc0:start();

for i = 0,DECCH do
    DecStart(i);
end

