
--[[

This script is used to decode 4 streams simultaneously 
and display on the screen. 

--]]
----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Number of decoded channels
DECCH       =       DECCH or 8;

-- Number of rendered images and their scaled size
OUTCH       =       OUTCH or 8;
FORMAT_OUT  =       FORMAT_OUT or "1080i";
FORMAT_IN   =       FORMAT_IN or "480i";
VIN         =       VIN or "svideo";
VOUT        =       VOUT or "component";
OUTW        =       OUTW or 480;
OUTH        =       OUTH or 480;

-- Width and height of the output grid
OUTGRIDW    =       OUTGRIDW or 4;
OUTGRIDH    =       OUTGRIDH or 2;

-- Input file names (# of files should match # of decoded channels)
INPUTFILENAME 	 = INPUTFILENAME or {};
INPUTFILENAME[0] = INPUTFILENAME[0] or "/directmnt/qa/streams/merlin/ibrbp480i.qbx";
INPUTFILENAME[1] = INPUTFILENAME[1] or "/directmnt/qa/streams/merlin/ibrbp480i.qbx";
INPUTFILENAME[2] = INPUTFILENAME[2] or "/directmnt/qa/streams/merlin/ibrbp480i.qbx";
INPUTFILENAME[3] = INPUTFILENAME[3] or "/directmnt/qa/streams/merlin/ibrbp480i.qbx";
INPUTFILENAME[4] = INPUTFILENAME[4] or "/directmnt/qa/streams/merlin/ibrbp480i.qbx";
INPUTFILENAME[5] = INPUTFILENAME[5] or "/directmnt/qa/streams/merlin/ibrbp480i.qbx";
INPUTFILENAME[6] = INPUTFILENAME[6] or "/directmnt/qa/streams/merlin/ibrbp480i.qbx";
INPUTFILENAME[7] = INPUTFILENAME[7] or "/directmnt/qa/streams/merlin/ibrbp480i.qbx";

-- Direct memory transfer mode (aka AVC instant display mode)
MEMORY_XFER_MODE = MEMORY_XFER_MODE or 1;

-- Play repeatly 
REPEAT_MODE = REPEAT_MODE or 1;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

if (MEMORY_XFER_MODE == 1) then
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC_INSTANT_DISPLAY;
else
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC;
end

videoInputBufferSize = 500 * 2048;
audioInputBufferSize = 50 * 2048;

BITSTREAM_OUTPUT = 	BITSTREAM_OUTPUT or 1;

sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=1, videocontrol=0x00101000, interleave=1});
sysctl0:setVPortMode({port=1, live=0});

sysctl0:configure({param=Q_SYS_CFG_DECODER_INPUT_PARTITION_SIZE, value=DECCH*videoInputBufferSize+DECCH*audioInputBufferSize});

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

board.setVideoOutput({port=0, format=FORMAT_OUT, mode=VOUT, sync=vout_sync});
board.setVideoInput({port=0, format=FORMAT_IN, mode=VIN});

sysctl0:setVerboseLevel({param=1});

COMPWIDTH=OUTGRIDW*OUTW;
COMPHEIGHT=OUTGRIDH*OUTH;
COMPYOFFSET= (1080-(OUTGRIDH*OUTH))/2;
COMPXOFFSET= (1920-(OUTGRIDW*OUTW))/2;

-- show or hide bitrate 0 for off, 1 for on
SHOWRATE = 0;

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
avoc0:setOutput1080i60({sync=vout_sync});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=COMPXOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000});
avoc0:activateCfg();

-- Create Decoders
avdec = {};
avsndec = {};
DECCH = DECCH - 1;

for i = 0,DECCH do
    local audioStreamId = 2*i + 1;
    local videoStreamId = 2*i + 2;

    avdec[i] = sysctl0:avdecCreate({name="avdec", atype=Q_AVDEC_CFP_AUDIO_CODEC_QMA, vtype=VIDEO_CODEC, audioBufferSize=audioInputBufferSize, videoBufferSize=videoInputBufferSize, maxWidth=720, maxHeight=480, numoutputframes=8, videoAUs=32});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=audioStreamId});
    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=videoStreamId});
    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_DISABLE_OUTPUT, value=0});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=24000});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_STEREO});
    if(REPEAT_MODE == 1) then
        avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_OVERRIDE, value=1});
    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_DEFAULT, value=3000});
    end
    avdec[i]:activateCfg();
end

vch = {};
x = 0;
y = 0;
OUTCH = OUTCH - 1;

for i = 0,OUTCH do
    avsndec[i] = sysctl0:avsnCreate({name="avsndec"});
    
    vch[i] = avoc0:addVideoChannel();
    
    avsndec[i]:bindVideoInput({src=avdec[i]});
       
    avoc0:setVideoChannelSource({ch=vch[i], src=avsndec[i]});
    
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=OUTW});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=OUTH});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});
    
    x = x + OUTW;
    if x == COMPWIDTH then
        x = 0;
        y = y + OUTH;
    end
end

avoc0:activateVideoChannelCfg();

bsr = {};
rs0 = sysctl0:createReadStreamer({name="rs0"});


function q()
    for i = 0,DECCH do
        avdec[i]:query();
    end
end


function avsnq()
    for i = 0,OUTCH do
        avsndec[i]:query();
    end
end


function DecStart(i)
    local audioStreamId = 2*i + 1;
    local videoStreamId = 2*i + 2;
    
    rs0:bindDecoder({dec=avdec[i]});
    bsr[i] = rs0:open({type="qbox", file=INPUTFILENAME[i], srcId={1, 2}, codecId={audioStreamId, videoStreamId}});
    rs0:setReaderParam({bsr=bsr[i], param=Q_RS_CFG_BSR_LOOPMODE, value=REPEAT_MODE});
    rs0:setReaderParam({bsr=bsr[i], param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
    avsndec[i]:start();
    avdec[i]:start();
    rs0:play({bsr=bsr[i]});
end


function DecStop(i)
    rs0:close({bsr=bsr[i]});
    avdec[i]:stop();
    avdec[i]:waitForVideoEOS();
    avsndec[i]:stop();
    rs0:unbindDecoder({dec=avdec[i]});
end

function StartStopCh(Ch, time)
    os.execute("sleep "..time);
    DecStop(Ch);
    DecStart(Ch);
end

function StartStopTest()
    while 1 do
        for i = 0, DECCH do
            StartStopCh(i, 7);
        end
    end
end



-- START
avoc0:start();

for i = 0,DECCH do
    DecStart(i);
end

