--[[
    
    script to demonstrate object tracking in following ways:
      1. Display results on display output using graphics supported by DirectFB.
      2. Send object tracking info in metadata which is streamed using RTP.
      3. Both of the above at the same time.

    The above are controlled using the options:

    OBJECTTRACKING_DISP 
    ENABLE_ANALYTICS_STREAMING

    If script is invoked with just mobiapp -f, then only analytic streaming would be there
    which is case (2) above.

    To display analytics on local display, do not use mobiapp -f to run this script directly,
    because it needs to load dfbobjtack.so. Instead, use ./obj.sh to launch it. By default, 
    this is case (1) above as obj.sh will set ENABLE_ANALYTICS_STREAMING to 0;
    For case (3), invoke obj.sh with argument "-a 1".


--]]

-- Video input mode
-- "svideo" 
-- "composite" 
-- "hdmi" 
-- "component"
VIN             = VIN or "composite";

-- Video output mode
-- "svideo" SVIDEO input/output (note composite is driven at same time on output)
VOUT            = VOUT or "svideo";

-- Video format for capture and display.
-- "480i"
-- "576i"
-- "1080i"
-- "720p"
FORMAT          = FORMAT or "480i";

-- Controls network streaming configuration
-- 0: no streaming
-- 2: rtp
STREAMING = STREAMING or 2;


-- Enable Analytics Streaming
-- 0 : Don't stream analytics
-- 1 : Stream analytics as RTP metadata with rtpwriter
-- 2 : Stream analytics in Private SEI with rtpwriter
ENABLE_ANALYTICS_STREAMING = ENABLE_ANALYTICS_STREAMING or 1 

-- RTP/RTSP Settings
-- ip address of rtp server.
SERVER_IP = SERVER_IP or "NULL";
-- port of rtp server
SERVER_PORT = SERVER_PORT or 554;
-- session identifier for rtp server
SESSION_ID = SESSION_ID or "test_D1";
-- rtp server mode
SERVER_MODE = SERVER_MODE or 0;
-- Maximum number of allowed client connections
MAX_CLIENTS_ALLOWED = MAX_CLIENTS_ALLOWED or 100;	--Putting a large number so that we do not impose any limit.
                                                        --For testing, a small number can be specified.

-- Multicast IP, make it NULL for unicast
MULTICAST_IP = MULTICAST_IP or "NULL";
-- Multicast Video Port
MULTICAST_VID_PORT = MULTICAST_VID_PORT or 4567;
-- Multicast Audio Port
MULTICAST_AUD_PORT = MULTICAST_AUD_PORT or 1234;

-- Set 1 to create third party client
STREAM_TO_THIRD_PARTY_CLIENT = STREAM_TO_THIRD_PARTY_CLIENT or 0;

-- Third party client Ip address
THIRD_PARTY_CLIENT_IP = THIRD_PARTY_CLIENT_IP or "10.32.84.197"

-- Third party client Port
THIRD_PARTY_CLIENT_PORT = THIRD_PARTY_CLIENT_PORT or 2048;

-- HTTP Port number
HTTP_PORT = HTTP_PORT or 8080;

-- End of RTP/RTSP Settings


HALF_FRAMERATE = HALF_FRAMERATE or 0;
-- Enable Local Analytics Display
OBJECTTRACKING_DISP = OBJECTTRACKING_DISP or 0

-- Video bitrate
if (FORMAT == "720p") or (FORMAT == "720p50") then
  VBITRATE = VBITRATE   or 6000000
elseif (FORMAT == "1080i")  or (FORMAT == "1080i50") then
  VBITRATE = VBITRATE or 6000000
elseif (FORMAT == "480i") or (FORMAT == "576i") then
  VBITRATE = VBITRATE or 2000000
end

GOP_STRUCTURE = "IP";
ENC_OUTPUT_BUFFER = VBITRATE / 2;
ENC_GUARD_BAND_PER_MB = 200;

if (FORMAT == "720p") then
        HSIZE         = 1280;  
        VSIZE         = 720;
        FRAME_STRUCTURE = "frame";
        MAX_PME_DOWNSCALE = 4;
        TF_LUMA_BYPASS = 0;
        TF_CHROMA_BYPASS = 1;
        if (GOP_STRUCTURE == "IP") then
                NUM_VIN_FRAMES = 8;
                NUM_AVC_REFERENCES = 8;
                AVSN_DELAY = 3;
        else
                NUM_VIN_FRAMES = 10;
                NUM_AVC_REFERENCES = 10;
                AVSN_DELAY = 8;
        end
        VIN_STC = 1500;
        VIN_OUTPUT_STC = 1500;
        AVC_LEVEL = 40;
        if (VIN == "component") then
                HSTART_PIXEL = 291;
        else
                HSTART_PIXEL = 251;
        end
        if (ENC_OUTPUT_BUFFER < 2000000) then
                ENC_OUTPUT_BUFFER = 2000000;
        end
elseif (FORMAT == "1080i") then
        HSIZE = 1920;  
        VSIZE = 1080;
        MAX_PME_DOWNSCALE = 4;
        TF_LUMA_BYPASS = 0;
        TF_CHROMA_BYPASS = 1;
        VIN_STC = 3003;
        VIN_OUTPUT_STC = 3003;
        ENC_OUTPUT_BUFFER = ENC_OUTPUT_BUFFER + VBITRATE / 4;
        if (GOP_STRUCTURE == "IP") then
                NUM_VIN_FRAMES = 8;
                NUM_AVC_REFERENCES = 6;
                AVSN_DELAY = 2;
        else
                NUM_VIN_FRAMES = 11;
                NUM_AVC_REFERENCES = 8;
                AVSN_DELAY = 8;
        end
                AVC_LEVEL = 40;
        if (VIN == "component") then
                HSTART_PIXEL = 235;
        else
                HSTART_PIXEL = 193;
        end
        if (ENC_OUTPUT_BUFFER < 2000000) then
                ENC_OUTPUT_BUFFER = 2000000;
        end
        vout_sync='external'
elseif (FORMAT == "480i") then
        HSIZE = 720;
        VSIZE = 480;
        MAX_PME_DOWNSCALE = 1;
        NUM_VIN_FRAMES = 9;
        TF_LUMA_BYPASS = 0;
        TF_CHROMA_BYPASS = 0;
        NUM_AVC_REFERENCES = 8;
        VIN_STC = 3003;
        VIN_OUTPUT_STC = 3003;
        if (GOP_STRUCTURE == "IP") then
                AVSN_DELAY = 3;
        else
                AVSN_DELAY = 4;
        end
                AVC_LEVEL = 31;
        if (FRAME_FORMAT == "frame") then
                AVC_LEVEL = 30;
        end
        if (ENC_OUTPUT_BUFFER < 1000000) then
                ENC_OUTPUT_BUFFER = 1000000;
        end
elseif (FORMAT == "576i") then
        HSIZE = 720;  
        VSIZE = 576;
        MAX_PME_DOWNSCALE = 1;
        NUM_VIN_FRAMES = 10;
        TF_LUMA_BYPASS = 0;
        TF_CHROMA_BYPASS = 0;
        NUM_AVC_REFERENCES = 8;
        VIN_STC = 3600;
        VIN_OUTPUT_STC = 3600;
        if (GOP_STRUCTURE == "IP") then
                AVSN_DELAY = 4;
        else
                AVSN_DELAY = 4;
        end
                AVC_LEVEL = 31;
        if (FRAME_FORMAT == "frame") then
                AVC_LEVEL = 30;
        end
        if (ENC_OUTPUT_BUFFER < 1000000) then
                ENC_OUTPUT_BUFFER = 1000000;
        end
else
        error("Invalid format " .. tostring(FORMAT))
end

if (HALF_FRAMERATE ~= 0) then
    VIN_OUTPUT_STC = VIN_OUTPUT_STC * 2;
    AVSN_DELAY = AVSN_DELAY - 1;
end

-- OSD_BUFFER_WIDTH is the physical width of the buffer in pixels which is
-- typically 2048/bytes per pixe if 16bpp
OSD_BUFFER_WIDTH        = OSD_BUFFER_WIDTH or 2048;
OSD_BUFFER_HEIGHT       = OSD_BUFFER_HEIGHT or VSIZE;
OSD_X_OFFSET            = OSD_X_OFFSET or 0;
OSD_Y_OFFSET            = OSD_Y_OFFSET or 0;
-- "triple buffer" mode, minimal buffer size is 2048*VSIZE*3
if (FORMAT == "720p") or (FORMAT == "1080i") then
    OSD_MEMORY_SIZE     = 8*1024*1024;
else
    OSD_MEMORY_SIZE     = 4*1024*1024;
end

OSD_DISPLAY_WIDTH       = OSD_DISPLAY_WIDTH or HSIZE;
OSD_DISPLAY_HEIGHT      = OSD_DISPLAY_HEIGHT or VSIZE;

--output file(s)
OUTFILE_H4V = OUTFILE_H4V or nil; -- "/data/test.h4v";   --"/path/test.h4v"
OUTFILE_QBOX = OUTFILE_QBOX or nil; --"/data/test.qbx"; --"/path/test.qbx"

-- modulo 16 size
VSIZE16 = 16*math.ceil(VSIZE/16)

VIDEOSTREAMID = 2;

sysctl0:setVerboseLevel({param=1});

board.setVideoInput({port=0, format=FORMAT, mode=VIN});

if (FORMAT == "1080i") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, sync=vout_sync});
  sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00000ba0, interleave=1});
elseif (FORMAT == "720p") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
  sysctl0:setVPortMode({port=0, live=1, interlaced=0, bottomfieldfirst=0, videocontrol=0x00000ba8, interleave=1});
elseif (FORMAT == "480i") or (FORMAT == "576i") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
  sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});
end

lvpp0 = sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=HSIZE, maxHeight=VSIZE16, 
                            numVidFrame=NUM_VIN_FRAMES, numPMEDSFrame=NUM_VIN_FRAMES, 
                            maxPMEDSFactor=MAX_PME_DOWNSCALE});
        
ws = sysctl0:createWriteStreamer({name="ws0"});
ws:start()

if (ENABLE_ANALYTICS_STREAMING > 0) then
	print("Enable metadata....");
	ws:configure({param=Q_WS_CFG_METADATA, value=1});
end

  -- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
if (FORMAT == "1080i") then
        Setup_VPP(lvpp0, HSIZE, VSIZE, 1, 0, 4, 4)
        lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=563});
        lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=20});
        lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=HSTART_PIXEL});
        lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
        lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=1});
        lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=1});
elseif (FORMAT == "720p") then
        Setup_VPP(lvpp0, HSIZE, VSIZE, 0, 0, 4, 4);
        lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=25});
        lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=HSTART_PIXEL});
        --lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=20});
        lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=1});
        lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=1});
elseif (FORMAT == "480i") then
        Setup_VPP(lvpp0, HSIZE, VSIZE, 1, 0, 2, 4)
        lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=263});
        lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
        lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=22});
        lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
        lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=8});
        lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=9}); 
elseif (FORMAT == "576i") then
        Setup_VPP(lvpp0, HSIZE, VSIZE, 1, 0, 2, 4)
        lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=313});
        lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
        lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=24});
        lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
        lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=16});
        lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=15}); 
        lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=VIN_STC});
        lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=VIN_OUTPUT_STC});
end

lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTY, value=TF_LUMA_BYPASS});
lvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTC, value=TF_CHROMA_BYPASS});
filterHorzEdgeAdapt(lvpp0, 13, 8, 4); 
filterTemporalMotion(lvpp0, 0, 0, 16, 240, 5, 5, 5, 5, 0);

lvpp0:activateCfg();

--avcenc0 = sysctl0:avcencCreate({name="avcenc0", ws=ws, maxwidth=HSIZE, maxheight=VSIZE16, numreferenceframes=4});
avcenc0 = sysctl0:avcencCreate({name="avcenc0", ws=ws, maxwidth=HSIZE, maxheight=VSIZE16, numreferenceframes=NUM_AVC_REFERENCES, outputbuffersize=ENC_OUTPUT_BUFFER, objectTrackingEnable=1});

if (FORMAT == "1080i") then
  avcenc_1080i60(avcenc0, "high", AVC_LEVEL, VBITRATE, Q_AVCENC_CFP_GOP_STRUCTURE_IP, "mbaff")
  avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_INVERSE_TELECINE_ENABLE, value=0});
elseif (FORMAT == "720p") then
    avcenc_720p60(avcenc0, "high", AVC_LEVEL, VBITRATE, Q_AVCENC_CFP_GOP_STRUCTURE_IP, "frame")
elseif (FORMAT == "480i") then
  if (HIGHQUALITY == 1) then
    avcenc_480i_highquality(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, Q_AVCENC_CFP_GOP_STRUCTURE_IP, "mbaff")
  else
    avcenc_480i(avcenc0, "high", AVC_LEVEL, VBITRATE, Q_AVCENC_CFP_GOP_STRUCTURE_IP, mbaff)
  end
elseif (FORMAT == "576i") then
  -- The 480i settings work for 576i except for the clock ticks
  if (HIGHQUALITY == 1) then
    avcenc_480i_highquality(avcenc0, PROFILE, AVC_LEVEL, VBITRATE, Q_AVCENC_CFP_GOP_STRUCTURE_IP, "mbaff") --mbaff?
  else
    avcenc_480i(avcenc0, "high", AVC_LEVEL, VBITRATE, Q_AVCENC_CFP_GOP_STRUCTURE_IP, "mbaff")
  end
  avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=VIN_OUTPUT_STC});
end


avoc0 = sysctl0:avocCreate({name="avoc0", gfx0Size=OSD_MEMORY_SIZE, allocGfx=1});
avsn0 = sysctl0:avsnCreate({name="avsn0"});

  if (FORMAT == "1080i") then
    avoc0:setOutput1080i60({sync=vout_sync});
  elseif (FORMAT == "720p") then
    avoc0:setOutput720p60();
  elseif (FORMAT == "480i") then
    avoc0:setOutput480i();
  elseif (FORMAT == "576i") then
    avoc0:setOutput576i();
  end

if (HALF_FRAMERATE ~= 0) then
  avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=VIN_OUTPUT_STC});
end

avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_ENABLE    , value=0});
avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW             , value=0});
avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT            , value=1});

-- object tracking parameter settings
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_ENABLE,                      value=1});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_DOWN_SAMPLE_FACTOR,          value=1});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_DOWN_SAMPLE_OFFSET,          value=0});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD,           value=12});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MIN,       value=9});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MAX,       value=15});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MULTFACT_MIN,       value=256});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MULTFACT_MAX,       value=256});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_LUMAMOT_THRESHOLD,           value=9});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_MINISIZE_X,                  value=30});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_MINISIZE_Y,                  value=30});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_MEDIAN_FILTER_ENABLE,        value=0});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_MOTION_VECTOR_ENABLE,        value=0});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_USE_DOWNSCALED_MOTION_VECTOR,value=0});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_STATIC_MB_THRESH,            value=5});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_STATIC_NUMFRAMES_THRESH,     value=300});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_UPDATE_TIMECONSTANT,      value=255});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_UPDATE_TIME,              value=30});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_APPEARANCETIME_THRESH,       value=2});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_CONTOUR_GENERATION_ENABLE,   value=1});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BLOBTRACKER_ENABLE,          value=1});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_FORCE_FIRST_FRAME,        value=1});
-- end object tracking parameter settings

--avcenc0:addVAPrivZone({width=45, height=30, offsetX=0, offsetY=0});
--avcenc0:subVAPrivZone({width=43, height=26, offsetX=1, offsetY=1});

avcenc0:activateVideoEncCfg();
avcenc0:bindVideoInput({src=lvpp0});

if (OUTFILE_H4V) then
  ws:open({type="h4v", file=OUTFILE_H4V, sid = {VIDEOSTREAMID}});
end
if (OUTFILE_QBOX) then
  ws:open({type="file", file=OUTFILE_QBOX, sid = {VIDEOSTREAMID}});
end

-- Streaming is done only in case of Video only
if(STREAMING == 2) then
  if (STREAM_TO_THIRD_PARTY_CLIENT == 1) then
    ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=none,mod=%d,msz=1472,mcli=%d,cli=1,cip=%s,cpt=%d,aim=%d", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE, MAX_CLIENTS_ALLOWED, THIRD_PARTY_CLIENT_IP, THIRD_PARTY_CLIENT_PORT, ENABLE_ANALYTICS_STREAMING), flags = Q_WS_CFP_FLAG_ENABLE_METADATA, sid = {VIDEOSTREAMID}});
  else
    ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=none,mod=%d,msz=1472,mcli=%d,mip=%s,mptv=%d,htp=%d,aim=%d",
	  SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE, MAX_CLIENTS_ALLOWED, MULTICAST_IP,
	  MULTICAST_VID_PORT, HTTP_PORT, ENABLE_ANALYTICS_STREAMING),
	  flags = Q_WS_CFP_FLAG_ENABLE_METADATA,
      sid = {VIDEOSTREAMID}});
  end          
end

avcenc0:setStreamID({streamid=VIDEOSTREAMID});
avsn0:bindVideoInput({src=lvpp0});
vch0 = avoc0:addVideoChannel();
avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
avoc0:activateVideoChannelCfg();

if (STREAMING > 0) then
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_SEI_PICT_TIMING_ENABLE, value=1});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});

    -- VLC does not support field coding so we have to turn off bottom field
    -- coding on intra frames
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_IP_FIELD_CODING, value=0});
    avcenc0:activateVideoEncCfg();
end

local ZOOM = 2
if(OBJECTTRACKING_DISP == 1) then
  osdinit({avoc=avoc0,width=OSD_DISPLAY_WIDTH,height=OSD_DISPLAY_HEIGHT,x=OSD_X_OFFSET,y=OSD_Y_OFFSET,memory=OSD_MEMORY_SIZE});
  if (FORMAT == "1080i") then
          os.execute("fbset 1920x1080 LUT8");  -- LUT8 format
  elseif (FORMAT == "720p") then
          os.execute("fbset 1280x720 LUT8");  -- LUT8 format
  elseif (FORMAT == "480i") then
          os.execute("fbset 720x480 LUT8");  -- LUT8 format
  elseif (FORMAT == "576i") then
          os.execute("fbset 720x576 LUT8");  -- LUT8 format
  end
  os.execute("fbset enable 1");   
  os.execute("fbset ck_mode 1");
  os.execute("fbset alpha_select 0");
  os.execute(string.format("fbset hzoom %d", ZOOM))
  os.execute(string.format("fbset vzoom %d", ZOOM))

  enableTracking({avcenc=avcenc0["handle"], xoff=0, yoff=0, width=HSIZE, height=VSIZE, width_org=HSIZE, height_org=VSIZE});
  
  -- Setting up trip wire
  setupTripwire({id=0, start_x=720/2, start_y=480/4, end_x=720/2, end_y=(480*3)/4, direction=0});
  enableTripwire({id=0, enable=1});
  
  --- Setting up ROI
  local Vertices = { {360,240}, {560,240}, {660,340}, {560,440}, {360,440} };
  --local Vertices = { {360,240}, {560,240}, {560,440}, {360,440} };
  setupROI({id=0, verts=Vertices});
  enableROI({id=0, enable=1});
  
  enableCapacity({id=0, enable=1, hysteresis=200});
  setupTracking({zoom=ZOOM, rect_thick=1, fill_contour=0, bitmap_contour=0});
end


avoc0:start();
avcenc0:record();
avsn0:start();
lvpp0:start();

function done()
        lvpp0:flush();
        waitForEvents(
        {objid=avcenc0, timeout=1000000, any=0, events={Q_AVCENC_EV_BITSTREAM_FLUSHED}});
        os.execute("echo flushed");

        if (OUTFILE_H4V) then
            ws:close({type="h4v", file=OUTFILE_H4V});
        end
        if (OUTFILE_QBOX) then    
            ws:close({type="file", file=OUTFILE_QBOX});
        end
end
