--[[

This script first decodes a 480i MPEG2 video stream, then re-encode it into
AVC format.  The transcoded stream will be shown on display.

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Input Filename
INPUTFILENAME = INPUTFILENAME or "/directmnt/qa/streams/merlin/hbo_dtc_sd.ts" ;
--INPUTFILENAME 	= FILENAME or "/home/fliu/svnproject/test_image/testcases/bitstreams/mpeg2/ts/hbo_dtc_sd.ts"

-- For QVGA output, set output to 320x240, AvcProfile = baseline
-- Input Size
InWidth 	= InWidth or 720;
InHeight 	= InHeight or 480;

-- Output Size
OutWidth 	= OutWidth or 720;
OutHeight	= OutHeight or 480;

-- AVC Profile: baseline or high
AvcProfile 	= AvcProfile or "high";

-- Display Enable
DisplayEnable = DisplayEnable or 1;

-- Audio/Video Stream IDs
-- NOTE: vidStreamId should always be 0 for rs to work
AudStreamId = AudStreamId or 1;
VidStreamId = VidStreamId or 0;

-- hbo_trailer is 17
STREAM_PID    = STREAM_PID or 17;

-- Loop Mode: Loop back to the beginning when file has reached the end
LoopMode 	= LoopMode or 1;

FORMAT 		= FORMAT or "480i";

-- Video bitrate
if (FORMAT == "720p") then
  VBITRATE 			= VBITRATE 			or 6000000
elseif (FORMAT == "1080i") then
  VBITRATE 			= VBITRATE 			or 6000000
elseif (FORMAT == "480i") or (FORMAT == "576i") then
  VBITRATE 			= VBITRATE 			or 2000000
end

VOUT		= VOUT or "svideo";

ENC_OUTPUT_BUFFER = VBITRATE / 2;
ENC_GUARD_BAND_PER_MB = 100;

PREVIEW =  PREVIEW or 2;

if (FORMAT == "720p") then
  HSIZE         = 1280;  
  VSIZE         = 720;
  FRAME_STRUCTURE = "frame"; -- override frame structure
  MAX_PME_DOWNSCALE = 4;
  TF_LUMA_BYPASS = 0;
  TF_CHROMA_BYPASS = 1;
  VIN_STC = 1500;
  VIN_OUTPUT_STC = 1500;
  if (GOP_STRUCTURE == "IP") then
    NUM_VIN_FRAMES = 8;
    NUM_AVC_REFERENCES = 8;
    AVSN_DELAY = 3;
  else
    NUM_VIN_FRAMES = 10;
    NUM_AVC_REFERENCES = 10;
    AVSN_DELAY = 8;
  end
  AVC_LEVEL = 40;
  if (VIN == "component") then
        HSTART_PIXEL = 261;
  else
        HSTART_PIXEL = 171;
  end
  if (ENC_OUTPUT_BUFFER < 2000000) then
    ENC_OUTPUT_BUFFER = 2000000;
    ENC_GUARD_BAND_PER_MB = 50;
  end
elseif (FORMAT == "1080i") then
  HSIZE = 1920;  
  VSIZE = 1080;
  MAX_PME_DOWNSCALE = 4;
  TF_LUMA_BYPASS = 0;
  TF_CHROMA_BYPASS = 1;
  VIN_STC = 3003;
  VIN_OUTPUT_STC = 3003;
  ENC_OUTPUT_BUFFER = ENC_OUTPUT_BUFFER + VBITRATE / 4;
  if (GOP_STRUCTURE == "IP") then
    NUM_VIN_FRAMES = 8;
    NUM_AVC_REFERENCES = 6;
    AVSN_DELAY = 2;
  else
    NUM_VIN_FRAMES = 11;
    NUM_AVC_REFERENCES = 8;
    AVSN_DELAY = 8;
  end
  AVC_LEVEL = 40;
   if (VIN == "component") then
        HSTART_PIXEL = 235;
  else
        HSTART_PIXEL = 193;
  end
  if (ENC_OUTPUT_BUFFER < 2000000) then
    ENC_OUTPUT_BUFFER = 2000000;
    ENC_GUARD_BAND_PER_MB = 50;
  end
elseif (FORMAT == "480i") then
  HSIZE = 720;  
  VSIZE = 480;
  MAX_PME_DOWNSCALE = 1;
  NUM_VIN_FRAMES = 9;
  TF_LUMA_BYPASS = 0;
  TF_CHROMA_BYPASS = 0;
  NUM_AVC_REFERENCES = 8;
  VIN_STC = 3003;
  VIN_OUTPUT_STC = 3003;
  if (GOP_STRUCTURE == "IP") then
    AVSN_DELAY = 3;
  else
    AVSN_DELAY = 4;
  end
  AVC_LEVEL = 31;
  if (FRAME_FORMAT == "frame") then
    AVC_LEVEL = 30;
  end
  if (ENC_OUTPUT_BUFFER < 1000000) then
    ENC_OUTPUT_BUFFER = 1000000;
    ENC_GUARD_BAND_PER_MB = 50;
  end
elseif (FORMAT == "576i") then
  HSIZE = 720;  
  VSIZE = 576;
  MAX_PME_DOWNSCALE = 1;
  NUM_VIN_FRAMES = 10;
  TF_LUMA_BYPASS = 0;
  TF_CHROMA_BYPASS = 0;
  NUM_AVC_REFERENCES = 8;
  VIN_STC = 3600;
  VIN_OUTPUT_STC = 3600;
  if (GOP_STRUCTURE == "IP") then
    AVSN_DELAY = 4;
  else
    AVSN_DELAY = 4;
  end
  AVC_LEVEL = 31;
  if (FRAME_FORMAT == "frame") then
    AVC_LEVEL = 30;
  end
  if (ENC_OUTPUT_BUFFER < 1000000) then
    ENC_OUTPUT_BUFFER = 1000000;
    ENC_GUARD_BAND_PER_MB = 50;
  end
else
  error("Invalid format " .. tostring(FORMAT))
end

-- Deinterlacing defaults to producing 60 fps output.  If it's being used, set the source
-- frame time to 60 fps, then set the target frame time to 30 fps so half the frames are dropped.
---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------


board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
sysctl0:setVPortMode({port=1, live=0});

sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ, value=27000000});
sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ, value=27000000});
sysctl0:configure({param=Q_SYS_CFG_VIDEO_INVERT_CONTROL, value=0x00000004});
sysctl0:configure({param=Q_SYS_CFG_SERIALIO_DRIVESTRENGTH_2, value=0x00002539});
sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL, value=0x0012aa00});

rs0 = sysctl0:createReadStreamer({name="rs0"});
avdec0 = sysctl0:avdecCreate({name="avdec0", atype=Q_AVDEC_CFP_AUDIO_CODEC_QMA, vtype=Q_AVDEC_CFP_VIDEO_CODEC_M2V, vstream=Q_AVDEC_CFP_VIDEO_STREAM_TS, audioBufferSize=100000, videoBufferSize=2000000, maxWidth=InWidth, maxHeight=InHeight, numoutputframes=17, videoAUs=64});
nvpp0 = sysctl0:nvppCreate({name="nvpp0", maxWidth=OutWidth, maxHeight=OutHeight});
ws = sysctl0:createWriteStreamer({name="ws0"});
avcenc0 = sysctl0:avcencCreate({name="avcenc0", ws=ws, maxwidth=OutWidth, maxheight=OutHeight, numreferenceframes=8});


-- AVDEC settings --


avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=AudStreamId});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=VidStreamId});
--avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_DISABLE_OUTPUT, value=Q_AVDEC_CFP_TRUE});
--avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_OVERFLOW_THRESHOLD, value=3000000});
--avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_OVERFLOW_MARGIN, value=20000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_THRESHOLD, value=1000000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_MARGIN, value=50000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_INITIAL_BUFFER_LEVEL, value=0});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_INPUT, value=Q_AVDEC_CFP_VIDEO_STREAM_FILE});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TS_PID, value=STREAM_PID});
avdec0:activateCfg();


-- NVPP settings --
if (AvcProfile == "baseline") then
    Setup_VPP(nvpp0, InWidth, InHeight, 0, 1, 2, 2);
    nvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=3003});
    nvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
    nvpp0:activateCfg();
else
    Setup_VPP(nvpp0, InWidth, InHeight, 1, 1, 2, 4);
end

nvpp0:crop({cropWidth=InWidth, cropHeight=InHeight, horzOffset=0, vertOffset=0});
nvpp0:size({outputWidth=OutWidth, outputHeight=OutHeight});

nvpp0:bindInput({src=avdec0});


-- AVCENC settings --


-- Set AVC high profile
if (AvcProfile == "high") then
    avcenc_480i(avcenc0, "high", 31, 4000000, Q_AVCENC_CFP_GOP_STRUCTURE_IBBRBP, "mbaff");
-- Set AVC baseline profile
elseif (AvcProfile == "baseline") then
    avcenc_480i(avcenc0, "baseline", 22, 200000, Q_AVCENC_CFP_GOP_STRUCTURE_IP, "frame");
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_PROFILE_BASELINE_COMPATIBLE, value=1});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_PROFILE_MAIN_COMPATIBLE, value=1});
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_ENABLE_8X8_TRANSFORM, value=0});
    for i = 0, 7, 1 do
        avcenc0:setVideoEncRegionParam({region=i,param=Q_AVCENC_REG_CMP_INTRAMODES_INTERTRANS_ISLICE, value=0});
        avcenc0:setVideoEncRegionParam({region=i,param=Q_AVCENC_REG_CMP_INTRAMODES_I_8X8_ISLICE, value=0});
        avcenc0:setVideoEncRegionParam({region=i,param=Q_AVCENC_REG_CMP_INTRAMODES_INTERTRANS_PSLICE, value=0});
        avcenc0:setVideoEncRegionParam({region=i,param=Q_AVCENC_REG_CMP_INTRAMODES_I_8X8_PSLICE, value=0});
    end
    avcenc0:activateVideoEncCfg();
end

--ws:open({type="h4v", file="test.h4v", sid = {VidStreamId}});
ws:open({type="h4v", file="/dev/null", sid = {VidStreamId}});
avcenc0:setStreamID({streamid=VidStreamId});
avcenc0:bindVideoInput({src=nvpp0});


-- Reconstructed preview --

--[[
if (DisplayEnable == 1) then
    avcenc_recpv_480i(avcenc0);
end
--]]

-- General audio/video preview setup
if ((PREVIEW == 1) or (PREVIEW == 2)) then
  avoc0 = sysctl0:avocCreate({name="avoc0"});
  avsn0 = sysctl0:avsnCreate({name="avsn0"});
  if (FORMAT == "1080i") then
    avoc0:setOutput1080i60({sync=vout_sync});
  elseif (FORMAT == "720p") then
    avoc0:setOutput720p60();
  elseif (FORMAT == "480i") then
    avoc0:setOutput480i();
  elseif (FORMAT == "576i") then
    avoc0:setOutput576i();
  end

  avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=HSIZE});
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=VSIZE});

  avoc0:activateCfg();

  vch0 = avoc0:addVideoChannel();
  avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=HSIZE});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=VSIZE});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=HSIZE});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=VSIZE});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});
  avoc0:activateVideoChannelCfg();

  -- Activate live or reconstructed video preview
  if (PREVIEW == 2) then
    avsn0:bindVideoInput({src=avcenc0});
 
    avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW , value=1});

    avsn0:setParam({param=Q_AVSN_CMP_EXTERNAL_STC_OFFSET, value=VIN_OUTPUT_STC * -1 * AVSN_DELAY});
  end
  avsn0:activateCfg();

  avcenc0:activateVideoEncCfg();
end


--avsn0:setParam({param=Q_AVSN_CMP_FIXED_LATENCY_ENABLE, value=1});
--avsn0:setParam({param=Q_AVSN_CMP_FIXED_LATENCY_TICKS, value=3000});
avsn0:activateCfg();

if ((PREVIEW == 1) or (PREVIEW == 2)) then
  avoc0:start();
  avsn0:start();
end


rs0:bindDecoder({dec=avdec0});

-- Start! --

bsr0 = rs0:open({type="dump", file=INPUTFILENAME, srcId={VidStreamId}});
rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_LOOPMODE, value=LoopMode});

ws:start();
avcenc0:record();
nvpp0:start();

avdec0:start();
rs0:play({bsr=bsr0});

-- Wait for stop command to finish
-- avdec0:waitForVideoEOS()

-- rs0:close({bsr=bsr0});
-- exit();
